import os
import requests
import json
from typing import List, Dict, Union, Generator, BinaryIO, Optional
from urllib.parse import urljoin
from ...utils.error_handler import (
    InvokeError,
    InvokeConnectionError,
    InvokeServerUnavailableError,
    InvokeRateLimitError,
    InvokeAuthorizationError,
    InvokeBadRequestError,
)
from ...utils.logger import logger
from ..base_api import BaseAPI, provider_specific

class API(BaseAPI):
    BASE_URL = "https://api.minimax.chat/v1/"

    def __init__(self, credentials: Dict[str, str]):
        super().__init__(credentials)
        self.api_key = credentials.get("api_key") or os.environ.get("MINIMAX_API_KEY")
        self.group_id = credentials.get("group_id") or os.environ.get("MINIMAX_GROUP_ID")
        if not self.api_key or not self.group_id:
            raise ValueError(
                "API key and Group ID must be provided either in credentials or as environment variables MINIMAX_API_KEY and MINIMAX_GROUP_ID")
        self.base_url = credentials.get("api_url", self.BASE_URL)
        self.session = requests.Session()
        self.session.headers.update({
            'Authorization': f'Bearer {self.api_key}',
            'Content-Type': 'application/json'
        })
        logger.info("MiniMax API initialized")
        logger.debug(f"Base URL: {self.BASE_URL}")

    def generate(self, model: str, messages: List[Dict[str, str]], **kwargs) -> Dict:
        """Generate a response using the specified model."""
        logger.info(f"Generating response with model: {model}")

        system_message = {
            "role": "system",
            "content": "MM智能助理是一款由MiniMax自研的，没有调用其他产品的接口的大型语言模型。MiniMax是一家中国科技公司，一直致力于进行大模型相关的研究。"
        }

        # 确保 messages 列表的第一条消息是 system 消息
        if messages[0].get('role') != 'system':
            messages = [system_message] + messages

        payload = {
            "model": model,
            "messages": messages,
            "tokens_to_generate": kwargs.get('tokens_to_generate', 2048),
            "temperature": kwargs.get('temperature', 0.01),
            "top_p": kwargs.get('top_p', 0.95),
        }
        return self._call_api("chat/completions", method="POST", json=payload)

    def stream_generate(self, model: str, messages: List[Dict[str, str]], **kwargs) -> Generator:
        """Generate a streaming response using the specified model."""
        logger.info(f"Generating streaming response with model: {model}")

        system_message = {
            "role": "system",
            "content": "MM智能助理是一款由MiniMax自研的，没有调用其他产品的接口的大型语言模型。MiniMax是一家中国科技公司，一直致力于进行大模型相关的研究。"
        }

        # 确保 messages 列表的第一条消息是 system 消息
        if messages[0].get('role') != 'system':
            messages = [system_message] + messages

        payload = {
            "model": model,
            "messages": messages,
            "tokens_to_generate": kwargs.get('tokens_to_generate', 2048),
            "temperature": kwargs.get('temperature', 0.01),
            "top_p": kwargs.get('top_p', 0.95),
            "stream": True
        }
        response = self._call_api("chat/completions", method="POST", json=payload, stream=True)
        for line in response.iter_lines():
            if line:
                line = line.decode('utf-8')
                if line.startswith('data: '):
                    try:
                        data = json.loads(line[6:])
                        if 'choices' in data and data['choices']:
                            delta = data['choices'][0].get('delta', {})
                            if 'content' in delta:
                                yield {'delta': {'text': delta['content']}}
                    except json.JSONDecodeError:
                        logger.error(f"Failed to parse streaming response: {line}")

    def create_embedding(self, model: str, texts: List[str], type: str) -> Dict:
        """Create embeddings for the given input."""
        logger.info(f"Creating embedding with model: {model}")
        payload = {
            "model": model,
            "texts": texts,
            "type": type
        }
        return self._call_api("embeddings", method="POST", json=payload)

    @provider_specific
    def text_to_speech(self, model: str, text: str, voice_setting: Dict, audio_setting: Dict, **kwargs) -> Dict:
        """Convert text to speech using T2A v2 API."""
        logger.info(f"Converting text to speech with model: {model}")
        payload = {
            "model": model,
            "text": text,
            "voice_setting": voice_setting,
            "audio_setting": audio_setting,
            **kwargs
        }
        return self._call_api("t2a_v2", method="POST", json=payload)

    @provider_specific
    def create_video_generation_task(self, model: str, prompt: str, **kwargs) -> Dict:
        """Create a video generation task."""
        logger.info(f"Creating video generation task with prompt: {prompt}")
        payload = {
            "model": model,
            "prompt": prompt,
            **kwargs
        }
        try:
            return self._call_api("video_generation", method="POST", json=payload)
        except InvokeRateLimitError:
            logger.warning("Rate limit reached for video generation task")
            return {"task_id": "", "base_resp": {"status_code": 1002, "status_msg": "rate limit"}}

    @provider_specific
    def query_video_generation_task(self, task_id: str) -> Dict:
        """Query the status of a video generation task."""
        logger.info(f"Querying video generation task: {task_id}")
        return self._call_api(f"query/video_generation", method="GET", params={"task_id": task_id})

    @provider_specific
    def upload_music(self, file: BinaryIO, purpose: str = 'song') -> Dict:
        """Upload a music file for voice or instrumental extraction."""
        logger.info(f"Uploading music file for purpose: {purpose}")

        files = {'file': (file.name, file, 'audio/mpeg')}
        data = {'purpose': purpose}

        try:
            response = self._call_api("music_upload", method="POST", files=files, data=data)
            logger.debug(f"Response: {response}")
            return response
        except InvokeError as e:
            logger.error(f"Music upload failed: {str(e)}")
            raise

    @provider_specific
    def generate_music(self, model: str, lyrics: str, refer_voice: Optional[str] = None,
                       refer_instrumental: Optional[str] = None, **kwargs) -> Dict:
        """Generate music based on lyrics and optional reference voice and instrumental."""
        logger.info(f"Generating music with model: {model}")
        payload = {
            "model": model,
            "lyrics": lyrics,
            "refer_voice": refer_voice,
            "refer_instrumental": refer_instrumental,
            **kwargs
        }
        return self._call_api("music_generation", method="POST", json=payload)

    def list_files(self, purpose: str = None) -> List[Dict]:
        """List files that have been uploaded to MiniMax."""
        logger.info("Listing files")
        params = {"purpose": purpose} if purpose else {}
        response = self._call_api("files/list", method="GET", params=params)
        return response.get('files', [])

    def upload_file(self, file: BinaryIO, purpose: str) -> Dict:
        """Upload a file to MiniMax."""
        logger.info(f"Uploading file for purpose: {purpose}")
        files = {'file': (file.name, file, 'application/octet-stream')}
        data = {'purpose': purpose}
        headers = {'Content-Type': None}  # Let requests set the correct Content-Type
        response = self._call_api("files/upload", method="POST", files=files, data=data, headers=headers)
        if isinstance(response, dict) and 'error' in response:
            raise InvokeError(f"File upload failed: {response['error']}")
        return response

    def delete_file(self, file_id: str) -> Dict:
        """Delete a file from MiniMax."""
        logger.info(f"Deleting file: {file_id}")
        return self._call_api(f"files/delete", method="POST", json={"file_id": file_id})

    def get_file_info(self, file_id: str) -> Dict:
        """Retrieve information about a specific file."""
        logger.info(f"Retrieving file info: {file_id}")
        return self._call_api(f"files/retrieve", method="GET", params={"file_id": file_id})

    def get_file_content(self, file_id: str) -> bytes:
        """Retrieve the content of a specific file."""
        logger.info(f"Retrieving file content: {file_id}")
        response = self._call_api(f"files/retrieve_content", method="GET", params={"file_id": file_id}, data={})
        return response

    def create_knowledge_base(self, name: str, embedding_model: str, operator_id: int, file_id: Optional[str] = None, **kwargs) -> Dict:
        """Create a new knowledge base."""
        logger.info(f"Creating knowledge base: {name}")
        payload = {
            "operator_id": operator_id,
            "name": name,
            "embedding_model": embedding_model,
            "file_id": file_id,
            **kwargs
        }
        return self._call_api("embedding/create_knowledge_base", method="POST", json=payload)

    def delete_knowledge_base(self, knowledge_base_id: str, operator_id: int) -> Dict:
        """Delete a knowledge base."""
        logger.info(f"Deleting knowledge base: {knowledge_base_id}")
        return self._call_api("embedding/delete_knowledge_base", method="POST", json={"knowledge_base_id": knowledge_base_id, "operator_id": operator_id})

    def get_knowledge_base(self, knowledge_base_id: str) -> Dict:
        """Get information about a specific knowledge base."""
        logger.info(f"Getting knowledge base: {knowledge_base_id}")
        return self._call_api(f"embedding/query_knowledge_base", method="GET", params={"knowledge_base_id": knowledge_base_id})

    def list_knowledge_bases(self, page: int = 0, page_size: int = 10) -> Dict:
        """List all knowledge bases."""
        logger.info("Listing knowledge bases")
        return self._call_api("embedding/list_knowledge_base", method="GET", params={"page": page, "page_size": page_size})

    def add_document_to_knowledge_base(self, knowledge_base_id: str, file_id: str, operator_id: int, **kwargs) -> Dict:
        """Add a document to a knowledge base."""
        logger.info(f"Adding document {file_id} to knowledge base {knowledge_base_id}")
        payload = {
            "knowledge_base_id": knowledge_base_id,
            "file_id": file_id,
            "operator_id": operator_id,
            **kwargs
        }
        return self._call_api("embedding/add_document", method="POST", json=payload)

    def delete_document_from_knowledge_base(self, knowledge_base_id: str, file_id: str, operator_id: int) -> Dict:
        """Delete a document from a knowledge base."""
        logger.info(f"Deleting document {file_id} from knowledge base {knowledge_base_id}")
        payload = {
            "knowledge_base_id": knowledge_base_id,
            "file_id": file_id,
            "operator_id": operator_id
        }
        return self._call_api("embedding/delete_document", method="POST", json=payload)

    def chatcompletion_pro(self, model: str, messages: List[Dict[str, str]], **kwargs) -> Dict:
        """Use the ChatCompletion Pro API."""
        logger.info(f"Using ChatCompletion Pro with model: {model}")
        payload = {
            "model": model,
            "messages": messages,
            "bot_setting": kwargs.get('bot_setting', [{"bot_name": "MM智能助理", "content": "MM智能助理是一款由MiniMax自研的大型语言模型。"}]),
            "reply_constraints": kwargs.get('reply_constraints', {"sender_type": "BOT", "sender_name": "MM智能助理"}),
            "tokens_to_generate": kwargs.get('tokens_to_generate', 2048),
            "temperature": kwargs.get('temperature', 0.01),
            "top_p": kwargs.get('top_p', 0.95),
            **kwargs
        }
        return self._call_api("text/chatcompletion_pro", method="POST", json=payload)

    def stream_chatcompletion_pro(self, model: str, messages: List[Dict[str, str]], **kwargs) -> Generator:
        """Use the ChatCompletion Pro API with streaming."""
        logger.info(f"Using streaming ChatCompletion Pro with model: {model}")

        payload = {
            "model": model,
            "messages": messages,
            "bot_setting": kwargs.get('bot_setting', [
                {
                    "bot_name": "MM智能助理",
                    "content": "MM智能助理是一款由MiniMax自研的，没有调用其他产品的接口的大型语言模型。MiniMax是一家中国科技公司，一直致力于进行大模型相关的研究。"
                }
            ]),
            "reply_constraints": kwargs.get('reply_constraints', {"sender_type": "BOT", "sender_name": "MM智能助理"}),
            "stream": True,
            "tokens_to_generate": kwargs.get('tokens_to_generate', 2048),
            "temperature": kwargs.get('temperature', 0.01),
            "top_p": kwargs.get('top_p', 0.95),
        }

        response = self._call_api("text/chatcompletion_pro", method="POST", json=payload, stream=True)

        for line in response.iter_lines():
            if line:
                chunk_str = line.decode("utf-8")
                logger.debug(f"Received chunk: {chunk_str}")
                if chunk_str.startswith("data: "):
                    try:
                        parsed_data = json.loads(chunk_str[6:])
                        if "usage" not in parsed_data:  # 忽略最后的使用情况信息
                            delta_content = parsed_data["choices"][0]["messages"]
                            yield {'delta': delta_content}
                    except json.JSONDecodeError:
                        logger.error(f"Failed to parse streaming response: {chunk_str}")

    @provider_specific
    def create_long_speech_task(self, model: str, text: str, voice_setting: Dict, audio_setting: Dict, **kwargs) -> Dict:
        """Create a long speech generation task using T2A Large v2 API."""
        logger.info(f"Creating long speech task with model: {model}")
        payload = {
            "model": model,
            "text": text,
            "voice_setting": voice_setting,
            "audio_setting": audio_setting,
            **kwargs
        }
        return self._call_api("t2a_async_v2", method="POST", json=payload)

    @provider_specific
    def query_long_speech_task(self, task_id: str) -> Dict:
        """Query the status of a long speech generation task."""
        logger.info(f"Querying long speech task: {task_id}")
        return self._call_api(f"query/t2a_async_query_v2", method="GET", params={"task_id": task_id})

    @provider_specific
    def voice_cloning(self, file_id: int, voice_id: str, **kwargs) -> Dict:
        """Clone a voice using Voice Cloning API."""
        logger.info(f"Cloning voice with file_id: {file_id}")
        payload = {
            "file_id": file_id,
            "voice_id": voice_id,
            **kwargs
        }
        return self._call_api("voice_clone", method="POST", json=payload)

    @provider_specific
    def text_to_voice(self, gender: str, age: str, voice_desc: List[str], text: str) -> Dict:
        """Generate a voice based on text description using Voice Generation API."""
        logger.info("Generating voice based on text description")
        payload = {
            "gender": gender,
            "age": age,
            "voice_desc": voice_desc,
            "text": text
        }
        return self._call_api("text2voice", method="POST", json=payload)

    @provider_specific
    def delete_voice(self, voice_type: str, voice_id: str) -> Dict:
        """Delete a voice using Delete_Voice API."""
        logger.info(f"Deleting voice with voice_id: {voice_id}")
        payload = {
            "voice_type": voice_type,
            "voice_id": voice_id
        }
        return self._call_api("delete_voice", method="POST", json=payload)

    def _call_api(self, endpoint: str, method: str = "POST", **kwargs):
        url = urljoin(self.base_url, endpoint)
        params = kwargs.pop('params', {})
        params['GroupId'] = self.group_id

        headers = self.session.headers.copy()
        if kwargs.get('stream'):
            headers['Accept'] = 'text/event-stream'
        if 'headers' in kwargs:
            headers.update(kwargs.pop('headers'))

        logger.debug(f"Sending request to {url}")
        logger.debug(f"Headers: {headers}")
        logger.debug(f"Method: {method}")
        logger.debug(f"Params: {params}")
        logger.debug(f"Kwargs: {json.dumps(kwargs, indent=2, default=str)}")

        try:
            if method == "GET":
                response = self.session.get(url, headers=headers, params=params, **kwargs)
            elif method == "POST":
                if 'files' in kwargs:
                    response = self.session.post(url, headers=headers, params=params, files=kwargs['files'], data=kwargs.get('data'))
                else:
                    response = self.session.post(url, headers=headers, params=params, json=kwargs.get('json'))
            else:
                raise ValueError(f"Unsupported HTTP method: {method}")

            logger.debug(f"Response status code: {response.status_code}")
            logger.debug(f"Response headers: {response.headers}")
            logger.debug(f"Trace-ID: {response.headers.get('Trace-Id')}")

            response.raise_for_status()

            if kwargs.get('raw_response'):
                return response.content
            elif kwargs.get('stream'):
                return response
            else:
                return response.json()
        except requests.RequestException as e:
            logger.error(f"Request failed: {str(e)}")
            if hasattr(e, 'response') and e.response is not None:
                logger.error(f"Error response status code: {e.response.status_code}")
                logger.error(f"Error response headers: {e.response.headers}")
                logger.error(f"Error response content: {e.response.text}")
            raise self._handle_error(e)

    def _handle_stream_response(self, response) -> Generator:
        logger.debug("Entering _handle_stream_response")
        for line in response.iter_lines():
            if line:
                line = line.decode('utf-8')
                logger.debug(f"Received line: {line}")
                if line.startswith('data: '):
                    try:
                        data = json.loads(line[6:])
                        logger.debug(f"Parsed data: {json.dumps(data, indent=2)}")
                        yield data
                    except json.JSONDecodeError:
                        logger.error(f"Failed to parse streaming response: {line}")
        logger.debug("Exiting _handle_stream_response")

    def _handle_error(self, error: requests.RequestException) -> InvokeError:
        if isinstance(error, requests.ConnectionError):
            return InvokeConnectionError(str(error))
        elif isinstance(error, requests.Timeout):
            return InvokeConnectionError(str(error))
        elif isinstance(error, requests.HTTPError):
            if error.response.status_code == 429:
                return InvokeRateLimitError(str(error))
            elif error.response.status_code in (401, 403):
                return InvokeAuthorizationError(str(error))
            elif error.response.status_code >= 500:
                return InvokeServerUnavailableError(str(error))
            else:
                return InvokeBadRequestError(str(error))
        else:
            return InvokeError(str(error))

    def set_proxy(self, proxy_url: str):
        """Set a proxy for API calls."""
        self.session.proxies = {
            'http': proxy_url,
            'https': proxy_url
        }
        logger.info(f"Proxy set to {proxy_url}")


from ..base_api import BaseAPI, provider_specific
from typing import List, Dict, Union, Generator
import requests
import json
import os
from urllib.parse import urljoin
from ...utils.logger import logger
from ...utils.error_handler import (
    InvokeError,
    InvokeConnectionError,
    InvokeServerUnavailableError,
    InvokeRateLimitError,
    InvokeAuthorizationError,
    InvokeBadRequestError,
)


class API(BaseAPI):
    BASE_URL = "https://api.moonshot.cn/v1/"

    def __init__(self, credentials: Dict[str, str]):
        super().__init__(credentials)
        self.api_key = credentials.get("api_key") or os.environ.get("MOONSHOT_API_KEY")
        if not self.api_key:
            raise ValueError(
                "API key must be provided either in credentials or as an environment variable MOONSHOT_API_KEY")
        self.base_url = credentials.get("api_url", self.BASE_URL)
        self.session = requests.Session()
        self.session.headers.update({
            'Authorization': f'Bearer {self.api_key}',
            'Content-Type': 'application/json'
        })
        logger.info("Kimi API initialized")
        logger.debug(f"Base URL: {self.BASE_URL}")

    def list_models(self) -> List[Dict]:
        """List available models."""
        logger.info("Fetching available models")
        response = self._call_api("models", method="GET")
        models = response.get('data', [])
        logger.info(f"Available models: {[model['id'] for model in models]}")
        return models

    @provider_specific
    def get_model(self, model_id: str) -> Dict:
        """Get information about a specific model."""
        logger.info(f"Fetching information for model: {model_id}")
        model_info = self._call_api(f"models/{model_id}", method="GET")
        logger.info(f"Model info for {model_id}: {model_info}")
        return model_info

    def generate(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]], **kwargs) -> Dict:
        """Generate a response using the specified model."""
        logger.info(f"Generating response with model: {model}")
        return self._call_api("chat/completions", model=model, messages=messages, **kwargs)

    def stream_generate(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]],
                        **kwargs) -> Generator:
        """Generate a streaming response using the specified model."""
        logger.info(f"Generating streaming response with model: {model}")
        kwargs['stream'] = True
        response = self._call_api("chat/completions", model=model, messages=messages, **kwargs)
        for chunk in response:
            if 'choices' in chunk and len(chunk['choices']) > 0:
                delta = chunk['choices'][0].get('delta', {})
                if 'content' in delta:
                    yield {'delta': {'text': delta['content']}}

    def count_tokens(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]]) -> int:
        """Count tokens in a message."""
        logger.info(f"Counting tokens for model: {model}")
        response = self._call_api("chat/completions", model=model, messages=messages, max_tokens=1)
        token_count = response['usage']['prompt_tokens']
        logger.info(f"Token count for model {model}: {token_count}")
        return token_count

    @provider_specific
    def create_cache(self, model: str, messages: List[Dict], tools: List[Dict] = None, name: str = None,
                     description: str = None, metadata: Dict[str, str] = None, expired_at: int = None,
                     ttl: int = None) -> Dict:
        """Create a context cache."""
        logger.info("Creating context cache")
        payload = {
            "model": model,
            "messages": messages,
            "tools": tools,
            "name": name,
            "description": description,
            "metadata": metadata,
        }
        if expired_at:
            payload["expired_at"] = expired_at
        elif ttl:
            payload["ttl"] = ttl

        return self._call_api("caching", method="POST", **payload)

    @provider_specific
    def list_caches(self, limit: int = 20, order: str = "desc", after: str = None, before: str = None,
                    metadata: Dict[str, str] = None) -> Dict:
        """List context caches."""
        logger.info("Listing context caches")
        params = {
            "limit": limit,
            "order": order,
            "after": after,
            "before": before,
        }
        if metadata:
            for key, value in metadata.items():
                params[f"metadata[{key}]"] = value

        return self._call_api("caching", method="GET", **params)

    @provider_specific
    def delete_cache(self, cache_id: str) -> Dict:
        """Delete a context cache."""
        logger.info(f"Deleting context cache: {cache_id}")
        return self._call_api(f"caching/{cache_id}", method="DELETE")

    @provider_specific
    def update_cache(self, cache_id: str, metadata: Dict[str, str] = None, expired_at: int = None,
                     ttl: int = None) -> Dict:
        """Update a context cache."""
        logger.info(f"Updating context cache: {cache_id}")
        payload = {}
        if metadata:
            payload["metadata"] = metadata
        if expired_at:
            payload["expired_at"] = expired_at
        elif ttl:
            payload["ttl"] = ttl

        return self._call_api(f"caching/{cache_id}", method="PUT", **payload)

    @provider_specific
    def get_cache(self, cache_id: str) -> Dict:
        """Get information about a specific context cache."""
        logger.info(f"Getting context cache: {cache_id}")
        return self._call_api(f"caching/{cache_id}", method="GET")

    @provider_specific
    def create_tag(self, tag: str, cache_id: str) -> Dict:
        """Create a tag for a context cache."""
        logger.info(f"Creating tag '{tag}' for cache: {cache_id}")
        return self._call_api("caching/refs/tags", method="POST", tag=tag, cache_id=cache_id)

    @provider_specific
    def list_tags(self, limit: int = 20, order: str = "desc", after: str = None, before: str = None) -> Dict:
        """List tags."""
        logger.info("Listing tags")
        params = {
            "limit": limit,
            "order": order,
            "after": after,
            "before": before,
        }
        return self._call_api("caching/refs/tags", method="GET", **params)

    @provider_specific
    def delete_tag(self, tag: str) -> Dict:
        """Delete a tag."""
        logger.info(f"Deleting tag: {tag}")
        return self._call_api(f"caching/refs/tags/{tag}", method="DELETE")

    @provider_specific
    def get_tag(self, tag: str) -> Dict:
        """Get information about a specific tag."""
        logger.info(f"Getting tag: {tag}")
        return self._call_api(f"caching/refs/tags/{tag}", method="GET")

    @provider_specific
    def get_tag_content(self, tag: str) -> Dict:
        """Get the context cache content for a specific tag."""
        logger.info(f"Getting content for tag: {tag}")
        return self._call_api(f"caching/refs/tags/{tag}/content", method="GET")

    def _call_api(self, endpoint: str, method: str = "POST", **kwargs):
        url = urljoin(self.base_url, endpoint)
        headers = self.session.headers.copy()

        if kwargs.get('stream'):
            headers['Accept'] = 'text/event-stream'

        logger.debug(f"Sending request to {url}")
        logger.debug(f"Headers: {headers}")
        logger.debug(f"Kwargs: {json.dumps(kwargs, indent=2)}")

        try:
            if method == "GET":
                response = self.session.get(url, params=kwargs, headers=headers)
            else:
                response = self.session.request(method, url, json=kwargs, headers=headers)

            logger.debug(f"Response status code: {response.status_code}")
            logger.debug(f"Response headers: {response.headers}")

            try:
                if not kwargs.get('stream'):
                    response_body = response.json()
                    logger.debug(f"Response body: {json.dumps(response_body, indent=2)}")
                else:
                    logger.debug("Streaming response, body not available")
            except json.JSONDecodeError:
                logger.debug(f"Response body (not JSON): {response.text}")

            response.raise_for_status()

            if kwargs.get('stream'):
                return self._handle_stream_response(response)
            else:
                return response.json()
        except requests.RequestException as e:
            logger.error(f"Request failed: {str(e)}")
            if hasattr(e, 'response') and e.response is not None:
                logger.error(f"Error response status code: {e.response.status_code}")
                logger.error(f"Error response headers: {e.response.headers}")
                try:
                    error_body = e.response.json()
                    logger.error(f"Error response body: {json.dumps(error_body, indent=2)}")
                except json.JSONDecodeError:
                    logger.error(f"Error response body (not JSON): {e.response.text}")
            raise self._handle_error(e)

    def _handle_stream_response(self, response) -> Generator:
        logger.debug("Entering _handle_stream_response")
        for line in response.iter_lines():
            if line:
                line = line.decode('utf-8')
                logger.debug(f"Received line: {line}")
                if line.startswith('data: '):
                    try:
                        data = json.loads(line[6:])
                        logger.debug(f"Parsed data: {json.dumps(data, indent=2)}")
                        yield data
                    except json.JSONDecodeError:
                        logger.error(f"Failed to parse streaming response: {line}")
        logger.debug("Exiting _handle_stream_response")

    def _handle_error(self, error: requests.RequestException) -> InvokeError:
        if isinstance(error, requests.ConnectionError):
            return InvokeConnectionError(str(error))
        elif isinstance(error, requests.Timeout):
            return InvokeConnectionError(str(error))
        elif isinstance(error, requests.HTTPError):
            if error.response.status_code == 429:
                return InvokeRateLimitError(str(error))
            elif error.response.status_code in (401, 403):
                return InvokeAuthorizationError(str(error))
            elif error.response.status_code >= 500:
                return InvokeServerUnavailableError(str(error))
            else:
                return InvokeBadRequestError(str(error))
        else:
            return InvokeError(str(error))

    def set_proxy(self, proxy_url: str):
        """Set a proxy for API calls."""
        self.session.proxies = {
            'http': proxy_url,
            'https': proxy_url
        }
        logger.info(f"Proxy set to {proxy_url}")


from ..base_api import BaseAPI
from typing import List, Dict, Union, Generator, Optional
import requests
import json
from ...utils.logger import logger
from ...utils.error_handler import (
    InvokeError,
    InvokeConnectionError,
    InvokeServerUnavailableError,
    InvokeRateLimitError,
    InvokeAuthorizationError,
    InvokeBadRequestError,
)

class API(BaseAPI):
    BASE_URL = "http://localhost:11434"

    def __init__(self, credentials: Dict[str, str]):
        super().__init__(credentials)
        self.base_url = credentials.get("base_url", self.BASE_URL)
        self.session = requests.Session()
        self.session.headers.update({
            'Content-Type': 'application/json'
        })
        logger.info("Ollama API initialized")

    def list_models(self) -> List[Dict]:
        """List available models."""
        logger.info("Fetching available models")
        response = self._call_api("/api/tags", method="GET")
        models = response.get('models', [])
        logger.info(f"Available models: {[model['name'] for model in models]}")
        return models

    def get_model(self, model_id: str) -> Dict:
        """Get information about a specific model."""
        logger.info(f"Fetching information for model: {model_id}")
        response = self._call_api(f"/api/show", method="POST", json={"name": model_id})
        logger.info(f"Model info for {model_id}: {response}")
        return response

    def generate(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]], **kwargs) -> Dict:
        """Generate a response using the specified model."""
        logger.info(f"Generating response with model: {model}")
        return self._call_api("/api/generate", model=model, messages=messages, stream=False, **kwargs)

    def stream_generate(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]], **kwargs) -> Generator:
        """Generate a streaming response using the specified model."""
        logger.info(f"Generating streaming response with model: {model}")
        yield from self._call_api("/api/generate", model=model, messages=messages, stream=True, **kwargs)

    def count_tokens(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]]) -> int:
        """Count tokens in a message."""
        logger.info(f"Counting tokens for model: {model}")
        response = self._call_api("/api/tokenize", model=model, messages=messages)
        token_count = len(response.get('tokens', []))
        logger.info(f"Token count for model {model}: {token_count}")
        return token_count

    def create_embedding(self, model: str, input: Union[str, List[str]], **kwargs) -> Dict:
        """Create embeddings for the given input."""
        logger.info(f"Creating embedding with model: {model}")
        return self._call_api("/api/embeddings", model=model, input=input, **kwargs)

    def _call_api(self, endpoint: str, method: str = "POST", **kwargs):
        url = f"{self.base_url}{endpoint}"
        stream = kwargs.pop('stream', False)
        
        data = {
            "model": kwargs.get('model'),
            "prompt": self._prepare_prompt(kwargs.get('messages', [])),
            "stream": stream,
            **kwargs
        }

        logger.debug(f"Sending request to {url}")
        logger.debug(f"Request data: {json.dumps(data, indent=2)}")

        try:
            if method == "GET":
                response = self.session.get(url, params=kwargs.get('params'))
            else:
                response = self.session.post(url, json=data, stream=stream)

            response.raise_for_status()

            if stream:
                return self._handle_stream_response(response)
            else:
                return response.json()
        except requests.RequestException as e:
            logger.error(f"API call error: {str(e)}")
            raise self._handle_error(e)

    def _prepare_prompt(self, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]]) -> str:
        """Prepare prompt from messages."""
        prompt = ""
        for message in messages:
            role = message.get('role', '')
            content = message.get('content', '')
            if isinstance(content, list):
                content = ' '.join([item.get('text', '') for item in content if item.get('type') == 'text'])
            prompt += f"{role.capitalize()}: {content}\n"
        return prompt.strip()

    def _handle_stream_response(self, response):
        for line in response.iter_lines():
            if line:
                yield json.loads(line)

    def _handle_error(self, error: requests.RequestException) -> InvokeError:
        if isinstance(error, requests.ConnectionError):
            return InvokeConnectionError(str(error))
        elif isinstance(error, requests.Timeout):
            return InvokeConnectionError(str(error))
        elif isinstance(error, requests.HTTPError):
            if error.response.status_code == 429:
                return InvokeRateLimitError(str(error))
            elif error.response.status_code in (401, 403):
                return InvokeAuthorizationError(str(error))
            elif error.response.status_code >= 500:
                return InvokeServerUnavailableError(str(error))
            else:
                return InvokeBadRequestError(str(error))
        else:
            return InvokeError(str(error))

    def set_proxy(self, proxy_url: str):
        """Set a proxy for API calls."""
        self.session.proxies = {
            'http': proxy_url,
            'https': proxy_url
        }
        logger.info(f"Proxy set to {proxy_url}")


import os
import sys
import unittest
from typing import List, Dict
import time
from unittest.mock import patch

# 添加项目根目录到 Python 路径
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from llm_onesdk.core import OneSDK
from llm_onesdk.utils.error_handler import InvokeError, InvokeUnsupportedOperationError
from llm_onesdk.utils.logger import Logger, logger

class TestDoubaoAPI(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        Logger.set_debug_mode(True)
        logger.info("Setting up TestDoubaoAPI class")

        cls.api_key = os.environ.get("DOUBAO_API_KEY")
        if not cls.api_key:
            raise ValueError("Please set DOUBAO_API_KEY environment variable")

        cls.sdk = OneSDK("doubao", {"api_key": cls.api_key})
        cls.sdk.set_debug_mode(True)

        cls.default_model = "ep-20241225233943-vkjxr"  # 使用正确的 tokenization 模型

    def test_count_tokens(self):
        logger.info("\nTesting count_tokens for Doubao:")
        messages = [
            {"role": "user", "content": "天空为什么这么蓝"},
            {"role": "user", "content": "花儿为什么这么香"}
        ]
        try:
            token_count = self.sdk.count_tokens(self.default_model, messages)
            self.assertIsInstance(token_count, int)
            self.assertTrue(token_count > 0)
            logger.info(f"Doubao token count: {token_count}")
        except InvokeUnsupportedOperationError:
            logger.info("Doubao does not support token counting")
        except Exception as e:
            logger.error(f"Unexpected error during count_tokens for Doubao: {str(e)}")
            self.fail(f"Unexpected error during count_tokens for Doubao: {str(e)}")

    def test_generate(self):
        logger.info("\nTesting generate for Doubao:")
        messages = [{"role": "user", "content": "Count from 1 to 5."}]
        try:
            response = self.sdk.generate(self.default_model, messages)
            self.assertIsInstance(response, Dict)
            self.assertIn('choices', response)
            self.assertIn('message', response['choices'][0])
            logger.info(f"Doubao response: {response['choices'][0]['message']['content']}")
        except InvokeUnsupportedOperationError:
            logger.info("Doubao does not support text generation")
        except Exception as e:
            logger.error(f"Unexpected error during generate for Doubao: {str(e)}")
            self.fail(f"Unexpected error during generate for Doubao: {str(e)}")

    def test_stream_generate(self):
        logger.info("\nTesting stream_generate for Doubao:")
        messages = [{"role": "user", "content": "Count from 1 to 5."}]
        try:
            stream = self.sdk.stream_generate(model=self.default_model, messages=messages)
            full_response = ""
            chunk_count = 0
            start_time = time.time()
            timeout = 30  # 30 seconds timeout
            for chunk in stream:
                if time.time() - start_time > timeout:
                    logger.warning("Timeout reached for Doubao")
                    break
                chunk_count += 1
                self.assertIsInstance(chunk, Dict)
                self.assertIn('choices', chunk)
                self.assertIn('message', chunk['choices'][0])
                content = chunk['choices'][0]['message'].get('content', '')
                if content:
                    full_response += content
                    logger.info(f"Doubao chunk {chunk_count}: {content}")
            logger.info(f"\nDoubao full response: {full_response}")
            logger.info(f"Total chunks received: {chunk_count}")
            logger.info(f"Time taken: {time.time() - start_time:.2f} seconds")
        except InvokeUnsupportedOperationError:
            logger.info("Doubao does not support stream generation")
        except Exception as e:
            logger.error(f"Unexpected error during stream_generate for Doubao: {str(e)}")
            self.fail(f"Unexpected error during stream_generate for Doubao: {str(e)}")

    def test_list_models(self):
        logger.info("\nTesting list_models for Doubao:")
        try:
            models = self.sdk.list_models()
            self.assertIsInstance(models, List)
            self.assertTrue(len(models) > 0)
            logger.info(f"Doubao models: {models}")
        except InvokeUnsupportedOperationError:
            logger.info("Doubao does not support listing models")
        except Exception as e:
            logger.error(f"Unexpected error while listing models for Doubao: {str(e)}")
            self.fail(f"Unexpected error while listing models for Doubao: {str(e)}")

    def test_tokenize(self):
        logger.info("\nTesting tokenize for Doubao:")
        texts = ["天空为什么这么蓝", "花儿为什么这么香"]
        try:
            result = self.sdk.tokenize(self.default_model, texts)
            self.assertIsInstance(result, Dict)
            self.assertIn('data', result)
            self.assertEqual(len(result['data']), len(texts))

            for item in result['data']:
                self.assertIn('token_ids', item)
                self.assertIn('total_tokens', item)
                self.assertIn('offset_mapping', item)

            logger.info(f"Doubao tokenization result: {result}")
        except InvokeUnsupportedOperationError:
            logger.info("Doubao does not support tokenization")
        except Exception as e:
            logger.error(f"Unexpected error during tokenize for Doubao: {str(e)}")
            self.fail(f"Unexpected error during tokenize for Doubao: {str(e)}")

    @patch.object(OneSDK, 'generate')
    def test_error_handling(self, mock_generate):
        mock_generate.side_effect = InvokeError("Test error")
        with self.assertRaises(InvokeError):
            self.sdk.generate(self.default_model, [{"role": "user", "content": "Test"}])

    def test_set_model(self):
        logger.info("\nTesting set_model for Doubao:")
        try:
            sdk_with_model = self.sdk.set_model(self.default_model)
            self.assertEqual(sdk_with_model.current_model, self.default_model)
            logger.info(f"Doubao model set successfully: {self.default_model}")
        except Exception as e:
            logger.error(f"Unexpected error during set_model for Doubao: {str(e)}")
            self.fail(f"Unexpected error during set_model for Doubao: {str(e)}")

if __name__ == "__main__":
    unittest.main(verbosity=2)


import os
import requests
import json
from typing import List, Dict, Union, Generator, BinaryIO
from urllib.parse import urljoin
from ...utils.error_handler import (
    InvokeError,
    InvokeConnectionError,
    InvokeServerUnavailableError,
    InvokeRateLimitError,
    InvokeAuthorizationError,
    InvokeBadRequestError,
)
from ...utils.logger import logger
from ..base_api import BaseAPI, provider_specific

class API(BaseAPI):
    BASE_URL = "https://api.openai.com/"

    def __init__(self, credentials: Dict[str, str]):
        super().__init__(credentials)
        self.api_key = credentials.get("api_key") or os.environ.get("OPENAI_API_KEY")
        if not self.api_key:
            raise ValueError(
                "API key must be provided either in credentials or as an environment variable OPENAI_API_KEY")
        self.base_url = credentials.get("api_url", self.BASE_URL)
        self.session = requests.Session()
        self.session.headers.update({
            'Authorization': f'Bearer {self.api_key}',
            'Content-Type': 'application/json'
        })
        logger.info("OpenAI API initialized")

    def list_models(self) -> List[Dict]:
        """List available models."""
        logger.info("Fetching available models")
        response = self._call_api("models", method="GET")
        models = response.get('data', [])
        logger.info(f"Available models: {[model['id'] for model in models]}")
        return models

    @provider_specific
    def get_model(self, model_id: str) -> Dict:
        """Get information about a specific model."""
        logger.info(f"Fetching information for model: {model_id}")
        model_info = self._call_api(f"models/{model_id}", method="GET")
        logger.info(f"Model info for {model_id}: {model_info}")
        return model_info

    def generate(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]], **kwargs) -> Dict:
        """Generate a response using the specified model."""
        logger.info(f"Generating response with model: {model}")
        return self._call_api("chat/completions", model=model, messages=messages, **kwargs)

    def stream_generate(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]],
                        **kwargs) -> Generator:
        """Generate a streaming response using the specified model."""
        logger.info(f"Generating streaming response with model: {model}")
        kwargs['stream'] = True
        return self._call_api("chat/completions", model=model, messages=messages, **kwargs)

    def create_completion(self, model: str, prompt: str, **kwargs) -> Dict:
        """Create a completion using the legacy API."""
        logger.info(f"Creating completion with model: {model}")
        return self._call_api("completions", model=model, prompt=prompt, **kwargs)

    def create_embedding(self, model: str, input: Union[str, List[str]], **kwargs) -> Dict:
        """Create embeddings for the given input."""
        logger.info(f"Creating embedding with model: {model}")
        return self._call_api("embeddings", model=model, input=input, **kwargs)

    @provider_specific
    def create_image(self, prompt: str, **kwargs) -> Dict:
        """Create an image based on the prompt."""
        logger.info(f"Creating image with prompt: {prompt}")
        return self._call_api("images/generations", prompt=prompt, **kwargs)

    @provider_specific
    def create_edit(self, image: BinaryIO, mask: BinaryIO, prompt: str, **kwargs) -> Dict:
        """Create an edit of an image based on a prompt."""
        logger.info(f"Creating image edit with prompt: {prompt}")
        files = {
            'image': ('image.png', image, 'image/png'),
            'mask': ('mask.png', mask, 'image/png')
        }
        data = {'prompt': prompt, **kwargs}
        return self._call_api("images/edits", method="POST", files=files, data=data)

    @provider_specific
    def create_variation(self, image: BinaryIO, **kwargs) -> Dict:
        """Create a variation of an image."""
        logger.info("Creating image variation")
        files = {'image': ('image.png', image, 'image/png')}
        return self._call_api("images/variations", method="POST", files=files, data=kwargs)

    @provider_specific
    def create_transcription(self, file: BinaryIO, model: str, **kwargs) -> Dict:
        """Transcribe audio to text."""
        logger.info(f"Creating transcription with model: {model}")
        files = {'file': ('audio.mp3', file, 'audio/mpeg')}
        data = {'model': model, **kwargs}
        return self._call_api("audio/transcriptions", method="POST", files=files, data=data)

    @provider_specific
    def create_translation(self, file: BinaryIO, model: str, **kwargs) -> Dict:
        """Translate audio to English text."""
        logger.info(f"Creating translation with model: {model}")
        files = {'file': ('audio.mp3', file, 'audio/mpeg')}
        data = {'model': model, **kwargs}
        return self._call_api("audio/translations", method="POST", files=files, data=data)

    @provider_specific
    def create_speech(self, model: str, input: str, voice: str, **kwargs) -> bytes:
        """Generate speech from text."""
        logger.info(f"Creating speech with model: {model}")
        data = {'model': model, 'input': input, 'voice': voice, **kwargs}
        return self._call_api("audio/speech", method="POST", data=data, raw_response=True)

    @provider_specific
    def create_moderation(self, input: Union[str, List[str]], **kwargs) -> Dict:
        """Create a moderation for the given input."""
        logger.info("Creating moderation")
        return self._call_api("moderations", input=input, **kwargs)

    def list_files(self) -> List[Dict]:
        """List files that have been uploaded to OpenAI."""
        logger.info("Listing files")
        response = self._call_api("files", method="GET")
        files = response.get('data', [])
        logger.info(f"Retrieved {len(files)} files")
        return files

    def upload_file(self, file: BinaryIO, purpose: str) -> Dict:
        """Upload a file to OpenAI."""
        logger.info(f"Uploading file for purpose: {purpose}")
        files = {'file': file}
        data = {'purpose': purpose}
        return self._call_api("files", method="POST", files=files, data=data)

    def delete_file(self, file_id: str) -> Dict:
        """Delete a file from OpenAI."""
        logger.info(f"Deleting file: {file_id}")
        return self._call_api(f"files/{file_id}", method="DELETE")

    def get_file_info(self, file_id: str) -> Dict:
        """Retrieve information about a specific file."""
        logger.info(f"Retrieving file info: {file_id}")
        return self._call_api(f"files/{file_id}", method="GET")

    def get_file_content(self, file_id: str) -> bytes:
        """Retrieve the content of a specific file."""
        logger.info(f"Retrieving file content: {file_id}")
        return self._call_api(f"files/{file_id}/content", method="GET", raw_response=True)

    @provider_specific
    def create_fine_tuning_job(self, training_file: str, model: str, **kwargs) -> Dict:
        """Create a fine-tuning job."""
        logger.info(f"Creating fine-tuning job for model: {model}")
        data = {'training_file': training_file, 'model': model, **kwargs}
        return self._call_api("fine_tuning/jobs", method="POST", json=data)

    @provider_specific
    def list_fine_tuning_jobs(self, **kwargs) -> Dict:
        """List fine-tuning jobs."""
        logger.info("Listing fine-tuning jobs")
        return self._call_api("fine_tuning/jobs", method="GET", params=kwargs)

    @provider_specific
    def get_fine_tuning_job(self, job_id: str) -> Dict:
        """Get info about a fine-tuning job."""
        logger.info(f"Getting info for fine-tuning job: {job_id}")
        return self._call_api(f"fine_tuning/jobs/{job_id}", method="GET")

    @provider_specific
    def cancel_fine_tuning_job(self, job_id: str) -> Dict:
        """Cancel a fine-tuning job."""
        logger.info(f"Cancelling fine-tuning job: {job_id}")
        return self._call_api(f"fine_tuning/jobs/{job_id}/cancel", method="POST")

    @provider_specific
    def list_fine_tuning_events(self, job_id: str, **kwargs) -> Dict:
        """List fine-tuning events for a job."""
        logger.info(f"Listing events for fine-tuning job: {job_id}")
        return self._call_api(f"fine_tuning/jobs/{job_id}/events", method="GET", params=kwargs)

    def _call_api(self, endpoint: str, method: str = "POST", **kwargs):
        url = urljoin(self.base_url, 'v1/' + endpoint)
        headers = self.session.headers.copy()

        logger.debug(f"Sending request to {url}")
        logger.debug(f"Headers: {headers}")
        logger.debug(f"Method: {method}")
        logger.debug(f"Kwargs: {kwargs}")

        try:
            if method == "GET":
                response = self.session.get(url, params=kwargs.get('params'))
            elif method == "POST":
                if 'files' in kwargs:
                    response = self.session.post(url, files=kwargs['files'], data=kwargs.get('data'))
                else:
                    response = self.session.post(url, json=kwargs)
            elif method == "DELETE":
                response = self.session.delete(url)
            else:
                raise ValueError(f"Unsupported HTTP method: {method}")

            response.raise_for_status()

            if kwargs.get('raw_response'):
                return response.content
            elif kwargs.get('stream'):
                return self._handle_stream_response(response)
            else:
                return response.json()
        except requests.RequestException as e:
            logger.error(f"API call error: {str(e)}")
            if hasattr(e, 'response') and e.response is not None:
                logger.error(f"Response body: {e.response.text}")
            raise self._handle_error(e)

    def _handle_stream_response(self, response) -> Generator:
        logger.debug("Entering _handle_stream_response")
        for line in response.iter_lines():
            if line:
                logger.debug(f"Received line: {line.decode('utf-8')}")
                if line.decode('utf-8').strip() == "data: [DONE]":
                    logger.debug("Received DONE signal, ending stream")
                    break
                try:
                    yield json.loads(line.decode('utf-8').split('data: ')[1])
                except json.JSONDecodeError as e:
                    logger.warning(f"Failed to decode JSON: {e}")
                    continue  # Skip this line and continue with the next one
        logger.debug("Exiting _handle_stream_response")

    def _handle_error(self, error: requests.RequestException) -> InvokeError:
        if isinstance(error, requests.ConnectionError):
            return InvokeConnectionError(str(error))
        elif isinstance(error, requests.Timeout):
            return InvokeConnectionError(str(error))
        elif isinstance(error, requests.HTTPError):
            if error.response.status_code == 429:
                return InvokeRateLimitError(str(error))
            elif error.response.status_code in (401, 403):
                return InvokeAuthorizationError(str(error))
            elif error.response.status_code >= 500:
                return InvokeServerUnavailableError(str(error))
            else:
                return InvokeBadRequestError(str(error))
        else:
            return InvokeError(str(error))

    def set_proxy(self, proxy_url: str):
        """Set a proxy for API calls."""
        self.session.proxies = {
            'http': proxy_url,
            'https': proxy_url
        }
        logger.info(f"Proxy set to {proxy_url}")


import os
import sys
import unittest
from typing import Dict
import time
from unittest.mock import patch

# 添加项目根目录到 Python 路径
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from llm_onesdk.core import OneSDK
from llm_onesdk.utils.error_handler import InvokeError, InvokeBadRequestError, InvokeRateLimitError
from llm_onesdk.utils.logger import Logger, logger

class TestBaichuanAPI(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        Logger.set_debug_mode(True)
        logger.info("Setting up TestBaichuanAPI class")

        cls.api_key = os.environ.get("BAICHUAN_API_KEY")
        if not cls.api_key:
            raise ValueError("Please set BAICHUAN_API_KEY environment variable")

        cls.sdk = OneSDK("baichuan", {"api_key": cls.api_key})
        cls.sdk.set_debug_mode(True)

        cls.chat_model = "Baichuan2-53B"
        cls.embedding_model = "Baichuan-Text-Embedding"

    def setUp(self):
        time.sleep(1)  # 添加延迟以避免频率限制

    def test_generate(self):
        logger.info("\nTesting generate for Baichuan:")
        messages = [{"role": "user", "content": "Count from 1 to 5."}]
        response = self.sdk.generate(self.chat_model, messages)
        self.assertIsInstance(response, Dict)
        self.assertIn('choices', response)
        self.assertIn('message', response['choices'][0])
        logger.info(f"Baichuan response: {response['choices'][0]['message']['content']}")

    def test_stream_generate(self):
        logger.info("\nTesting stream_generate for Baichuan:")
        messages = [{"role": "user", "content": "Count from 1 to 5."}]
        stream = self.sdk.stream_generate(model=self.chat_model, messages=messages)
        full_response = ""
        chunk_count = 0
        start_time = time.time()
        timeout = 30  # 30 seconds timeout
        for chunk in stream:
            if time.time() - start_time > timeout:
                logger.warning("Timeout reached for Baichuan")
                break
            chunk_count += 1
            self.assertIsInstance(chunk, Dict)
            self.assertIn('choices', chunk)
            self.assertIn('delta', chunk['choices'][0])
            content = chunk['choices'][0]['delta'].get('content', '')
            if content:
                full_response += content
                logger.info(f"Baichuan chunk {chunk_count}: {content}")
        logger.info(f"\nBaichuan full response: {full_response}")
        logger.info(f"Total chunks received: {chunk_count}")
        logger.info(f"Time taken: {time.time() - start_time:.2f} seconds")

    def test_create_embedding_single(self):
        logger.info("\nTesting create_embedding (single input) for Baichuan:")
        input_text = "百川大模型"
        response = self.sdk.create_embedding(self.embedding_model, input_text)
        self.assertIsInstance(response, Dict)
        self.assertIn('data', response)
        self.assertEqual(len(response['data']), 1)
        self.assertIn('embedding', response['data'][0])
        self.assertIn('usage', response)
        logger.info(f"Baichuan embedding: {response['data'][0]['embedding'][:5]}...")
        logger.info(f"Usage: {response['usage']}")

    def test_create_embedding_batch(self):
        logger.info("\nTesting create_embedding (batch input) for Baichuan:")
        input_texts = ["新年快乐", "百川大模型"]
        response = self.sdk.create_embedding(self.embedding_model, input_texts)
        self.assertIsInstance(response, Dict)
        self.assertIn('data', response)
        self.assertEqual(len(response['data']), 2)
        for item in response['data']:
            self.assertIn('embedding', item)
        self.assertIn('usage', response)
        logger.info(f"Baichuan embeddings: {[item['embedding'][:5] for item in response['data']]}")
        logger.info(f"Usage: {response['usage']}")

    @patch.object(OneSDK, 'generate')
    def test_error_handling_generate(self, mock_generate):
        mock_generate.side_effect = InvokeError("Test error")
        with self.assertRaises(InvokeError):
            self.sdk.generate(self.chat_model, [{"role": "user", "content": "Test"}])

    @patch.object(OneSDK, 'create_embedding')
    def test_error_handling_embedding(self, mock_create_embedding):
        mock_create_embedding.side_effect = InvokeBadRequestError("Invalid input")
        with self.assertRaises(InvokeBadRequestError):
            self.sdk.create_embedding(self.embedding_model, "")  # 空输入应该触发错误

    @patch.object(OneSDK, 'create_embedding')
    def test_rate_limit_embedding(self, mock_create_embedding):
        mock_create_embedding.side_effect = InvokeRateLimitError("Rate limit exceeded")
        with self.assertRaises(InvokeRateLimitError):
            self.sdk.create_embedding(self.embedding_model, "Test rate limit")

if __name__ == "__main__":
    unittest.main(verbosity=2)


import os
import sys
import unittest
from typing import Dict
import time
from unittest.mock import patch

# 添加项目根目录到 Python 路径
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from llm_onesdk.core import OneSDK
from llm_onesdk.utils.error_handler import InvokeError, InvokeConnectionError, InvokeRateLimitError, InvokeAuthorizationError
from llm_onesdk.utils.logger import Logger, logger

class TestAnthropicAPI(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        Logger.set_debug_mode(True)
        logger.info("Setting up TestAnthropicAPI class")

        cls.api_key = os.environ.get("ANTHROPIC_API_KEY")
        if not cls.api_key:
            raise ValueError("Please set ANTHROPIC_API_KEY environment variable")

        cls.base_url = os.environ.get("ANTHROPIC_URL")
        cls.sdk = OneSDK("anthropic", {
            "api_key": cls.api_key,
            "api_url": cls.base_url
        })
        cls.sdk.set_debug_mode(True)

        cls.default_model = "claude-3-5-sonnet-20241022"

    def setUp(self):
        time.sleep(1)  # 添加延迟以避免频率限制

    def test_list_models(self):
        logger.info("\nTesting list_models for Anthropic:")
        models = self.sdk.list_models()
        self.assertIsInstance(models, Dict)
        self.assertTrue(len(models) > 0)
        logger.info(f"Anthropic models: {models}")

    def test_base_url(self):
        logger.info("\nTesting base URL for Anthropic:")
        self.assertEqual(self.sdk.api.base_url, self.base_url)
        logger.info(f"Base URL correctly set to: {self.base_url}")

    def test_get_model(self):
        logger.info("\nTesting get_model for Anthropic:")
        model_info = self.sdk.get_model(self.default_model)
        self.assertIsInstance(model_info, Dict)
        self.assertEqual(model_info['id'], self.default_model)
        logger.info(f"Anthropic model info: {model_info}")

    def test_generate(self):
        logger.info("\nTesting generate for Anthropic:")
        messages = [{"role": "user", "content": "Count from 1 to 5."}]
        response = self.sdk.generate(self.default_model, messages, max_tokens=100)
        self.assertIsInstance(response, Dict)
        self.assertIn('content', response)
        logger.info(f"Anthropic response: {response['content']}")

    def test_stream_generate(self):
        logger.info("\nTesting stream_generate for Anthropic:")
        messages = [{"role": "user", "content": "Count from 1 to 5."}]
        stream = self.sdk.stream_generate(model=self.default_model, messages=messages, max_tokens=100)
        full_response = ""
        chunk_count = 0
        start_time = time.time()
        timeout = 30  # 30 seconds timeout
        for chunk in stream:
            if time.time() - start_time > timeout:
                logger.warning("Timeout reached for Anthropic")
                break
            chunk_count += 1
            self.assertIsInstance(chunk, Dict)
            logger.info(f"Received chunk {chunk_count}: {chunk}")
            content = chunk.get('delta', {}).get('text', '')
            if content:
                full_response += content
                logger.info(f"Content: {content}")
        logger.info(f"\nAnthropic full response: {full_response}")
        logger.info(f"Total chunks received: {chunk_count}")
        logger.info(f"Time taken: {time.time() - start_time:.2f} seconds")
        self.assertTrue(chunk_count > 0, "No chunks were received")
        self.assertTrue(len(full_response) > 0, "No content was received")

    def test_count_tokens(self):
        logger.info("\nTesting count_tokens for Anthropic:")
        messages = [
            {"role": "user", "content": "Hello, how are you?"},
            {"role": "assistant", "content": "I'm doing well, thank you for asking. How can I assist you today?"}
        ]
        token_count = self.sdk.count_tokens(self.default_model, messages)
        self.assertIsInstance(token_count, int)
        self.assertTrue(token_count > 0)
        logger.info(f"Anthropic token count: {token_count}")

    @patch.object(OneSDK, 'generate')
    def test_error_handling_generate(self, mock_generate):
        mock_generate.side_effect = InvokeError("Test error")
        with self.assertRaises(InvokeError):
            self.sdk.generate(self.default_model, [{"role": "user", "content": "Test"}])

    @patch.object(OneSDK, 'generate')
    def test_connection_error(self, mock_generate):
        mock_generate.side_effect = InvokeConnectionError("Connection error")
        with self.assertRaises(InvokeConnectionError):
            self.sdk.generate(self.default_model, [{"role": "user", "content": "Test"}])

    @patch.object(OneSDK, 'generate')
    def test_rate_limit_error(self, mock_generate):
        mock_generate.side_effect = InvokeRateLimitError("Rate limit exceeded")
        with self.assertRaises(InvokeRateLimitError):
            self.sdk.generate(self.default_model, [{"role": "user", "content": "Test"}])

    @patch.object(OneSDK, 'generate')
    def test_authorization_error(self, mock_generate):
        mock_generate.side_effect = InvokeAuthorizationError("Invalid API key")
        with self.assertRaises(InvokeAuthorizationError):
            self.sdk.generate(self.default_model, [{"role": "user", "content": "Test"}])

    # def test_set_proxy(self):
    #     logger.info("\nTesting set_proxy for Anthropic:")
    #     proxy_url = "http://example.com:8080"  # 使用一个示例 URL，不要实际连接
    #     self.sdk.set_proxy(proxy_url)
    #     logger.info(f"Proxy set to {proxy_url}")
    #     # 注意：这里我们只是测试方法调用，不测试实际连接

if __name__ == "__main__":
    unittest.main(verbosity=2)


import logging
import os


class Logger:
    _instance = None

    def __new__(cls):
        if cls._instance is None:
            cls._instance = super(Logger, cls).__new__(cls)
            cls._instance._logger = logging.getLogger('OneSDK')
            cls._instance._logger.setLevel(logging.INFO)
            cls._instance._formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

            # Console handler
            ch = logging.StreamHandler()
            ch.setFormatter(cls._instance._formatter)
            cls._instance._logger.addHandler(ch)

        return cls._instance

    @classmethod
    def get_logger(cls):
        return cls()._logger

    @classmethod
    def set_debug_mode(cls, debug: bool = False):
        logger = cls.get_logger()
        if debug:
            logger.setLevel(logging.DEBUG)
        else:
            logger.setLevel(logging.INFO)


logger = Logger.get_logger()


from typing import Optional, Dict, Any

class InvokeError(Exception):
    """Base class for all invoke errors."""
    def __init__(self, message: str, error_code: Optional[str] = None, 
                 http_status: Optional[int] = None, details: Optional[Dict[str, Any]] = None):
        self.message = message
        self.error_code = error_code
        self.http_status = http_status
        self.details = details or {}
        super().__init__(self.message)

    def __str__(self):
        error_parts = [f"Error {self.error_code}: {self.message}" if self.error_code else self.message]
        if self.http_status:
            error_parts.append(f"HTTP Status: {self.http_status}")
        if self.details:
            error_parts.append(f"Details: {self.details}")
        return " | ".join(error_parts)

class InvokeConnectionError(InvokeError):
    """Raised when there's a connection error during the API call."""
    pass

class InvokeServerUnavailableError(InvokeError):
    """Raised when the server is unavailable."""
    pass

class InvokeRateLimitError(InvokeError):
    """Raised when the API rate limit is exceeded."""
    def __init__(self, message: str, retry_after: Optional[int] = None, **kwargs):
        super().__init__(message, **kwargs)
        self.retry_after = retry_after

    def __str__(self):
        base_str = super().__str__()
        if self.retry_after:
            return f"{base_str} | Retry after: {self.retry_after} seconds"
        return base_str

class InvokeAuthorizationError(InvokeError):
    """Raised when there's an authentication or authorization error."""
    pass

class InvokeBadRequestError(InvokeError):
    """Raised when the request is invalid or cannot be served."""
    pass

class InvokeTimeoutError(InvokeError):
    """Raised when the API request times out."""
    pass

class InvokeAPIError(InvokeError):
    """Raised for any other API-related errors not covered by the specific classes above."""
    pass

class InvokeModelNotFoundError(InvokeError):
    """Raised when the specified model is not found."""
    pass

class InvokeInvalidParameterError(InvokeError):
    """Raised when an invalid parameter is provided in the API call."""
    pass

class InvokeUnsupportedOperationError(InvokeError):
    """Raised when an unsupported operation is attempted."""
    pass

class InvokeConfigError(InvokeError):
    """Raised when there's a configuration error."""
    pass

def handle_api_error(error: Exception) -> InvokeError:
    """
    Convert provider-specific errors to our custom InvokeError types.
    This function should be implemented in each provider's specific API module.
    """
    # Default error handling
    error_message = str(error)
    http_status = getattr(error, 'status_code', None) if hasattr(error, 'status_code') else None

    if isinstance(error, ConnectionError):
        return InvokeConnectionError(f"Connection error occurred: {error_message}", 
                                     error_code="CONNECTION_ERROR", http_status=http_status)
    elif isinstance(error, TimeoutError):
        return InvokeTimeoutError(f"Request timed out: {error_message}", 
                                  error_code="TIMEOUT", http_status=http_status)
    else:
        return InvokeAPIError(f"API error occurred: {error_message}", 
                              error_code="UNKNOWN_ERROR", http_status=http_status)


from typing import Any, Dict, Optional

class Config:
    def __init__(self, initial_config: Optional[Dict[str, Any]] = None):
        self._config = initial_config or {}

    def get(self, key: str, default: Any = None) -> Any:
        """获取配置值"""
        return self._config.get(key, default)

    def set(self, key: str, value: Any) -> None:
        """设置配置值"""
        self._config[key] = value

    def update(self, new_config: Dict[str, Any]) -> None:
        """更新多个配置值"""
        self._config.update(new_config)

    def remove(self, key: str) -> None:
        """移除配置项"""
        self._config.pop(key, None)

    def clear(self) -> None:
        """清除所有配置"""
        self._config.clear()

    def __getitem__(self, key: str) -> Any:
        """允许使用字典样式访问配置"""
        return self._config[key]

    def __setitem__(self, key: str, value: Any) -> None:
        """允许使用字典样式设置配置"""
        self._config[key] = value

    def __contains__(self, key: str) -> bool:
        """检查配置是否存在"""
        return key in self._config

    def __repr__(self) -> str:
        """返回配置的字符串表示"""
        return f"Config({self._config})"

    @property
    def as_dict(self) -> Dict[str, Any]:
        """返回配置的字典副本"""
        return self._config.copy()

    def load_from_file(self, file_path: str) -> None:
        """从文件加载配置"""
        import json
        with open(file_path, 'r') as f:
            self._config.update(json.load(f))

    def save_to_file(self, file_path: str) -> None:
        """保存配置到文件"""
        import json
        with open(file_path, 'w') as f:
            json.dump(self._config, f, indent=2)

# 默认配置
DEFAULT_CONFIG = {
    'debug': False,
    'timeout': 30,
    'max_retries': 3,
    'retry_delay': 1,
}

# 全局配置实例
global_config = Config(DEFAULT_CONFIG)


from abc import ABC, abstractmethod
from typing import List, Dict, Union, Generator, Any, BinaryIO
import requests
import os
from ..utils.logger import logger
from ..utils.error_handler import (
    InvokeConnectionError,
    InvokeServerUnavailableError,
    InvokeRateLimitError,
    InvokeAuthorizationError,
    InvokeBadRequestError,
    InvokeUnsupportedOperationError,
)

def provider_specific(func):
    func._provider_specific = True
    return func

class BaseAPI(ABC):
    def __init__(self, credentials: Dict[str, str]):
        self.credentials = credentials
        self.session = requests.Session()
        # self.setup_credentials()

    @abstractmethod
    def generate(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]], **kwargs) -> Dict:
        """Generate a response using the specified model."""
        pass

    @abstractmethod
    def stream_generate(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]], **kwargs) -> Generator:
        """Generate a streaming response using the specified model."""
        pass

    def set_proxy(self, proxy_url: str):
        """Set a proxy for API calls."""
        self.session.proxies = {
            'http': proxy_url,
            'https': proxy_url
        }
        logger.info(f"Proxy set to {proxy_url}")

    def custom_operation(self, operation: str, **kwargs):
        """Perform a custom operation specific to this provider."""
        raise InvokeUnsupportedOperationError("Custom operations are not implemented for this provider")

    @classmethod
    def get_provider_specific_methods(cls):
        return [name for name, method in cls.__dict__.items() if getattr(method, '_provider_specific', False)]

    def _call_api(self, endpoint: str, method: str = "POST", **kwargs):
        """Base method for making API calls"""
        try:
            response = self.session.request(method, endpoint, **kwargs)
            response.raise_for_status()
            return response
        except requests.RequestException as e:
            self._handle_error(e)

    def _handle_error(self, error: requests.RequestException):
        """Handle common API errors"""
        if isinstance(error, requests.ConnectionError):
            raise InvokeConnectionError(str(error))
        elif isinstance(error, requests.Timeout):
            raise InvokeConnectionError(str(error))
        elif isinstance(error, requests.HTTPError):
            if error.response.status_code == 429:
                raise InvokeRateLimitError(str(error))
            elif error.response.status_code in (401, 403):
                raise InvokeAuthorizationError(str(error))
            elif error.response.status_code >= 500:
                raise InvokeServerUnavailableError(str(error))
            else:
                raise InvokeBadRequestError(str(error))
        else:
            raise InvokeBadRequestError(str(error))

    def _validate_messages(self, messages: List[Dict[str, Any]]):
        """Validate the format of input messages"""
        if not isinstance(messages, list):
            raise ValueError("Messages must be a list")
        for message in messages:
            if not isinstance(message, dict) or 'role' not in message or 'content' not in message:
                raise ValueError("Each message must be a dictionary with 'role' and 'content' keys")

    @staticmethod
    def _get_env_var(var_name: str, default: str = None) -> str:
        """Safely get an environment variable"""
        return os.environ.get(var_name, default)

    def _log_debug(self, message: str):
        """Log a debug message"""
        logger.debug(message)

    def _log_info(self, message: str):
        """Log an info message"""
        logger.info(message)

    def _log_error(self, message: str):
        """Log an error message"""
        logger.error(message)


import os
import requests
import json
from typing import List, Dict, Union, Generator
from urllib.parse import urljoin
from ...utils.error_handler import (
    InvokeError,
    InvokeConnectionError,
    InvokeServerUnavailableError,
    InvokeRateLimitError,
    InvokeAuthorizationError,
    InvokeBadRequestError,
)
from ...utils.logger import logger
from ..base_api import BaseAPI, provider_specific


class API(BaseAPI):
    BASE_URL = "https://api.baichuan-ai.com/v1/"

    def __init__(self, credentials: Dict[str, str]):
        super().__init__(credentials)
        self.api_key = credentials.get("api_key") or os.environ.get("BAICHUAN_API_KEY")
        if not self.api_key:
            raise ValueError(
                "API key must be provided either in credentials or as an environment variable BAICHUAN_API_KEY")
        self.base_url = credentials.get("api_url", self.BASE_URL)
        self.session = requests.Session()
        self.session.headers.update({
            'Authorization': f'Bearer {self.api_key}',
            'Content-Type': 'application/json'
        })
        logger.info("Baichuan API initialized")

    def generate(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]], **kwargs) -> Dict:
        """Generate a response using the specified model."""
        logger.info(f"Generating response with model: {model}")
        endpoint = "chat/completions" if model.startswith("Baichuan2") else "chat"
        return self._call_api(endpoint, method="POST", json={
            "model": model,
            "messages": messages,
            **kwargs
        })

    def stream_generate(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]],
                        **kwargs) -> Generator:
        """Generate a streaming response using the specified model."""
        logger.info(f"Generating streaming response with model: {model}")
        endpoint = "chat/completions" if model.startswith("Baichuan2") else "stream/chat"
        kwargs['stream'] = True
        response = self._call_api(endpoint, method="POST", json={
            "model": model,
            "messages": messages,
            **kwargs
        }, stream=True)
        return self._handle_stream_response(response)

    def create_embedding(self, model: str, input: Union[str, List[str]], **kwargs) -> Dict:
        """Create embeddings for the given input."""
        logger.info(f"Creating embedding with model: {model}")
        return self._call_api("embeddings", method="POST", json={
            "model": model,
            "input": input
        })

    def set_proxy(self, proxy_url: str):
        """Set a proxy for API calls."""
        self.session.proxies = {
            'http': proxy_url,
            'https': proxy_url
        }
        logger.info(f"Proxy set to {proxy_url}")

    def _call_api(self, endpoint: str, method: str = "POST", **kwargs):
        url = urljoin(self.base_url, endpoint)
        logger.debug(f"Sending request to {url}")
        logger.debug(f"Method: {method}")
        logger.debug(f"Headers: {self.session.headers}")  # 打印请求头
        logger.debug(f"Kwargs: {kwargs}")

        try:
            response = self.session.request(method, url, **kwargs)
            response.raise_for_status()

            if kwargs.get('raw_response'):
                return response.content
            elif kwargs.get('stream'):
                return response
            else:
                return response.json()
        except requests.RequestException as e:
            logger.error(f"API call error: {str(e)}")
            # 尝试输出响应体
            try:
                error_content = e.response.text if e.response else "No response content"
                logger.error(f"Error response content: {error_content}")
            except AttributeError:
                logger.error("Unable to retrieve error response content")
            raise self._handle_error(e)

    def _handle_stream_response(self, response) -> Generator:
        logger.debug("Entering _handle_stream_response")
        for line in response.iter_lines():
            if line:
                logger.debug(f"Received line: {line.decode('utf-8')}")
                line = line.decode('utf-8')
                if line.startswith("data: "):
                    line = line[6:]  # Remove "data: " prefix
                try:
                    yield json.loads(line)
                except json.JSONDecodeError:
                    logger.error(f"Failed to parse JSON: {line}")
        logger.debug("Exiting _handle_stream_response")

    def _handle_error(self, error: requests.RequestException) -> InvokeError:
        if isinstance(error, requests.ConnectionError):
            return InvokeConnectionError(str(error))
        elif isinstance(error, requests.Timeout):
            return InvokeConnectionError(str(error))
        elif isinstance(error, requests.HTTPError):
            if error.response.status_code == 429:
                return InvokeRateLimitError(str(error))
            elif error.response.status_code in (401, 403):
                return InvokeAuthorizationError(str(error))
            elif error.response.status_code >= 500:
                return InvokeServerUnavailableError(str(error))
            else:
                return InvokeBadRequestError(str(error))
        else:
            return InvokeError(str(error))



import os
import sys
import unittest
from typing import List, Dict
import time
from unittest.mock import patch

# 添加项目根目录到 Python 路径
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from llm_onesdk.core import OneSDK
from llm_onesdk.utils.error_handler import InvokeError, InvokeConnectionError, InvokeRateLimitError, InvokeAuthorizationError
from llm_onesdk.utils.logger import Logger, logger


class TestOpenAIAPI(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        Logger.set_debug_mode(True)
        logger.info("Setting up TestOpenAIAPI class")

        cls.api_key = os.environ.get("OPENAI_API_KEY")
        if not cls.api_key:
            raise ValueError("Please set OPENAI_API_KEY environment variable")
        cls.base_url = os.environ.get("OPENAI_URL")
        cls.sdk = OneSDK("openai", {"api_key": cls.api_key, "api_url": cls.base_url})
        cls.sdk.set_debug_mode(True)

        cls.default_model = "gpt-3.5-turbo"

    def setUp(self):
        time.sleep(1)  # 添加延迟以避免频率限制

    def test_list_models(self):
        logger.info("\nTesting list_models for OpenAI:")
        models = self.sdk.list_models()
        self.assertIsInstance(models, List)
        self.assertTrue(len(models) > 0)
        logger.info(f"OpenAI models: {models}")

    def test_get_model(self):
        logger.info("\nTesting get_model for OpenAI:")
        model_info = self.sdk.get_model(self.default_model)
        self.assertIsInstance(model_info, Dict)
        self.assertEqual(model_info['id'], self.default_model)
        logger.info(f"OpenAI model info: {model_info}")

    def test_generate(self):
        logger.info("\nTesting generate for OpenAI:")
        messages = [{"role": "user", "content": "Count from 1 to 5."}]
        response = self.sdk.generate(self.default_model, messages, max_tokens=100)
        self.assertIsInstance(response, Dict)
        self.assertIn('choices', response)
        logger.info(f"OpenAI response: {response['choices'][0]['message']['content']}")

    def test_stream_generate(self):
        logger.info("\nTesting stream_generate for OpenAI:")
        messages = [{"role": "user", "content": "Count from 1 to 5."}]
        stream = self.sdk.stream_generate(model=self.default_model, messages=messages, max_tokens=100)
        full_response = ""
        chunk_count = 0
        start_time = time.time()
        timeout = 30  # 30 seconds timeout
        for chunk in stream:
            if time.time() - start_time > timeout:
                logger.warning("Timeout reached for OpenAI")
                break
            chunk_count += 1
            self.assertIsInstance(chunk, Dict)
            logger.info(f"Received chunk {chunk_count}: {chunk}")
            content = chunk.get('choices', [{}])[0].get('delta', {}).get('content', '')
            if content:
                full_response += content
                logger.info(f"Content: {content}")
        logger.info(f"\nOpenAI full response: {full_response}")
        logger.info(f"Total chunks received: {chunk_count}")
        logger.info(f"Time taken: {time.time() - start_time:.2f} seconds")
        self.assertTrue(chunk_count > 0, "No chunks were received")
        self.assertTrue(len(full_response) > 0, "No content was received")

    def test_create_embedding(self):
        logger.info("\nTesting create_embedding for OpenAI:")
        input_text = "Hello, world!"
        response = self.sdk.api.create_embedding("text-embedding-ada-002", input_text)
        self.assertIsInstance(response, Dict)
        self.assertIn('data', response)
        self.assertTrue(len(response['data']) > 0)
        logger.info(f"OpenAI embedding created: {len(response['data'][0]['embedding'])} dimensions")

    def test_create_image(self):
        logger.info("\nTesting create_image for OpenAI:")
        prompt = "A cute baby sea otter"
        response = self.sdk.api.create_image(prompt=prompt, n=1, size="256x256")
        self.assertIsInstance(response, Dict)
        self.assertIn('data', response)
        self.assertTrue(len(response['data']) > 0)
        logger.info(f"OpenAI image created: {response['data'][0]['url']}")

    def test_create_moderation(self):
        logger.info("\nTesting create_moderation for OpenAI:")
        input_text = "I want to hurt someone."
        response = self.sdk.api.create_moderation(input=input_text)
        self.assertIsInstance(response, Dict)
        self.assertIn('results', response)
        self.assertTrue(len(response['results']) > 0)
        logger.info(f"OpenAI moderation result: {response['results'][0]['flagged']}")

    def test_list_files(self):
        logger.info("\nTesting list_files for OpenAI:")
        files = self.sdk.api.list_files()
        self.assertIsInstance(files, List)
        logger.info(f"OpenAI files: {files}")

    @patch.object(OneSDK, 'generate')
    def test_error_handling_generate(self, mock_generate):
        mock_generate.side_effect = InvokeError("Test error")
        with self.assertRaises(InvokeError):
            self.sdk.generate(self.default_model, [{"role": "user", "content": "Test"}])

    @patch.object(OneSDK, 'generate')
    def test_connection_error(self, mock_generate):
        mock_generate.side_effect = InvokeConnectionError("Connection error")
        with self.assertRaises(InvokeConnectionError):
            self.sdk.generate(self.default_model, [{"role": "user", "content": "Test"}])

    @patch.object(OneSDK, 'generate')
    def test_rate_limit_error(self, mock_generate):
        mock_generate.side_effect = InvokeRateLimitError("Rate limit exceeded")
        with self.assertRaises(InvokeRateLimitError):
            self.sdk.generate(self.default_model, [{"role": "user", "content": "Test"}])

    @patch.object(OneSDK, 'generate')
    def test_authorization_error(self, mock_generate):
        mock_generate.side_effect = InvokeAuthorizationError("Invalid API key")
        with self.assertRaises(InvokeAuthorizationError):
            self.sdk.generate(self.default_model, [{"role": "user", "content": "Test"}])

    # def test_set_proxy(self):
    #     logger.info("\nTesting set_proxy for OpenAI:")
    #     proxy_url = "http://example.com:8080"  # 使用一个示例 URL，不要实际连接
    #     self.sdk.set_proxy(proxy_url)
    #     logger.info(f"Proxy set to {proxy_url}")
    #     # 注意：这里我们只是测试方法调用，不测试实际连接

    def test_fine_tuning_workflow(self):
        # 1. 创建微调任务
        logger.info("\nTesting create_fine_tuning_job for OpenAI:")
        training_file = "file-abc123"  # 替换为实际的文件ID
        response = self.sdk.api.create_fine_tuning_job(training_file=training_file, model="gpt-3.5-turbo")
        self.assertIsInstance(response, Dict)
        self.assertIn('id', response)
        job_id = response['id']
        logger.info(f"Fine-tuning job created: {job_id}")

        # 等待一段时间，确保任务已经被处理
        time.sleep(5)

        # 2. 获取微调任务信息
        logger.info("\nTesting get_fine_tuning_job for OpenAI:")
        response = self.sdk.api.get_fine_tuning_job(job_id)
        self.assertIsInstance(response, Dict)
        self.assertEqual(response['id'], job_id)
        logger.info(f"Fine-tuning job info retrieved: {response['status']}")

        # 3. 列出微调事件
        logger.info("\nTesting list_fine_tuning_events for OpenAI:")
        response = self.sdk.api.list_fine_tuning_events(job_id)
        self.assertIsInstance(response, Dict)
        self.assertIn('data', response)
        logger.info(f"Fine-tuning events listed: {len(response['data'])} events")

        # 4. 取消微调任务
        logger.info("\nTesting cancel_fine_tuning_job for OpenAI:")
        response = self.sdk.api.cancel_fine_tuning_job(job_id)
        self.assertIsInstance(response, Dict)
        self.assertEqual(response['id'], job_id)
        logger.info(f"Fine-tuning job cancelled: {response['status']}")

    def test_list_fine_tuning_jobs(self):
        logger.info("\nTesting list_fine_tuning_jobs for OpenAI:")
        response = self.sdk.api.list_fine_tuning_jobs()
        self.assertIsInstance(response, Dict)
        self.assertIn('data', response)
        logger.info(f"Fine-tuning jobs listed: {len(response['data'])} jobs")


if __name__ == "__main__":
    unittest.main(verbosity=2)



import os
import sys
import unittest
from typing import Dict
import time
from unittest.mock import patch

# 添加项目根目录到 Python 路径
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from llm_onesdk.core import OneSDK
from llm_onesdk.utils.error_handler import InvokeError
from llm_onesdk.utils.logger import Logger, logger


class TestWenxinAPI(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        Logger.set_debug_mode(True)
        logger.info("Setting up TestWenxinAPI class")

        cls.api_key = os.environ.get("WENXIN_API_KEY")
        cls.secret_key = os.environ.get("WENXIN_SECRET_KEY")
        if not cls.api_key or not cls.secret_key:
            raise ValueError("Please set WENXIN_API_KEY and WENXIN_SECRET_KEY environment variables")

        cls.sdk = OneSDK("wenxin", {"api_key": cls.api_key, "secret_key": cls.secret_key})
        cls.sdk.set_debug_mode(True)

        # 设置自定义模型
        custom_endpoint = "/rpc/2.0/ai_custom/v1/wenxinworkshop/chat/custom_model"  # 确保这个端点是正确的
        cls.sdk.api.set_custom_model("Custom-Model", custom_endpoint)

        cls.default_model = "ERNIE-Bot"
        cls.custom_model = "Custom-Model"

    def test_count_tokens(self):
        logger.info("\nTesting count_tokens for Wenxin:")
        messages = [
            {"role": "user", "content": "你好，最近如何？"},
            {"role": "assistant", "content": "我很好，谢谢你的关心。今天我能为你做些什么呢？"}
        ]
        token_count = self.sdk.count_tokens(self.default_model, messages)
        self.assertIsInstance(token_count, int)
        self.assertTrue(token_count > 0)
        logger.info(f"Wenxin token count: {token_count}")

    def test_generate(self):
        logger.info("\nTesting generate for Wenxin:")
        messages = [{"role": "user", "content": "请从1数到5。"}]
        response = self.sdk.generate(self.default_model, messages)
        self.assertIsInstance(response, Dict)
        self.assertIn('choices', response)
        self.assertTrue(len(response['choices']) > 0, "Choices list is empty")
        self.assertIn('message', response['choices'][0])
        content = response['choices'][0]['message'].get('content', '')
        logger.info(f"Wenxin response: {content}")

        self.assertTrue(content, "Response content is empty")
        if "Error" in content:
            logger.warning(f"API returned an error response: {content}")
        else:
            self.assertIn("1", content)
            self.assertIn("5", content)

    def test_stream_generate(self):
        logger.info("\nTesting stream_generate for Wenxin:")
        messages = [{"role": "user", "content": "请从1数到5。"}]
        stream = self.sdk.stream_generate(model=self.default_model, messages=messages)
        full_response = ""
        chunk_count = 0
        start_time = time.time()
        timeout = 30  # 30 seconds timeout
        for chunk in stream:
            if time.time() - start_time > timeout:
                logger.warning("Timeout reached for Wenxin")
                break
            chunk_count += 1
            self.assertIsInstance(chunk, Dict)
            self.assertIn('choices', chunk)
            self.assertIn('delta', chunk['choices'][0])
            content = chunk['choices'][0]['delta'].get('content', '')
            if content:
                full_response += content
                logger.info(f"Wenxin chunk {chunk_count}: {content}")
        logger.info(f"\nWenxin full response: {full_response}")
        logger.info(f"Total chunks received: {chunk_count}")
        logger.info(f"Time taken: {time.time() - start_time:.2f} seconds")

        self.assertTrue(full_response, "Full response is empty")
        self.assertIn("1", full_response)
        self.assertIn("5", full_response)

    def test_custom_model(self):
        logger.info("\nTesting custom model for Wenxin:")
        messages = [{"role": "user", "content": "使用自定义模型。"}]
        response = self.sdk.generate(self.custom_model, messages)
        self.assertIsInstance(response, Dict)
        self.assertIn('choices', response)
        self.assertTrue(len(response['choices']) > 0, "Choices list is empty")
        self.assertIn('message', response['choices'][0])
        content = response['choices'][0]['message'].get('content', '')
        logger.info(f"Wenxin custom model response: {content}")

        if 'error' in response:
            logger.warning(f"Custom model returned an error: {response['error']}")
            self.skipTest(f"Custom model error: {response['error'].get('message', 'Unknown error')}")
        else:
            self.assertTrue(content, "Custom model response content is empty")

    @patch.object(OneSDK, 'generate')
    def test_error_handling(self, mock_generate):
        logger.info("\nTesting error handling for Wenxin:")
        mock_generate.side_effect = InvokeError("Test error")
        with self.assertRaises(InvokeError):
            self.sdk.generate(self.default_model, [{"role": "user", "content": "Test"}])
        logger.info("Error handling test passed")

    def test_long_conversation(self):
        logger.info("\nTesting long conversation for Wenxin:")
        messages = [
            {"role": "user", "content": "你是谁？"},
            {"role": "assistant", "content": "我是文心一言开发的人工智能助手。"},
            {"role": "user", "content": "你能做什么？"},
            {"role": "assistant", "content": "我可以回答问题、提供信息、进行对话等。"},
            {"role": "user", "content": "给我讲个笑话。"}
        ]
        response = self.sdk.generate(self.default_model, messages)
        self.assertIsInstance(response, Dict)
        self.assertIn('choices', response)
        self.assertTrue(len(response['choices']) > 0, "Choices list is empty")
        content = response['choices'][0]['message'].get('content', '')
        logger.info(f"Wenxin long conversation response: {content}")
        self.assertTrue(content, "Long conversation response is empty")

    def test_embedding(self):
        logger.info("\nTesting embedding for Wenxin:")
        input_text = "这是一个测试文本"
        response = self.sdk.api.embedding("Embedding-V1", input_text)
        self.assertIsInstance(response, Dict)
        self.assertIn('data', response)
        self.assertTrue(len(response['data']) > 0)
        self.assertIn('embedding', response['data'][0])
        logger.info(f"Wenxin embedding response: {response}")

    def test_text_to_image(self):
        logger.info("\nTesting text to image for Wenxin:")
        prompt = "一只可爱的猫咪"
        response = self.sdk.api.text_to_image("Stable-Diffusion-XL", prompt)
        self.assertIsInstance(response, Dict)
        self.assertIn('data', response)
        self.assertTrue(len(response['data']) > 0)
        self.assertIn('b64_image', response['data'][0])
        logger.info(f"Wenxin text to image response received")

    def test_create_service(self):
        logger.info("\nTesting create service for Wenxin:")
        response = self.sdk.api.create_service("Test Service", 123, "test_uri")
        self.assertIsInstance(response, Dict)
        self.assertIn('result', response)
        self.assertIn('serviceId', response['result'])
        logger.info(f"Wenxin create service response: {response}")

    def test_get_service_detail(self):
        logger.info("\nTesting get service detail for Wenxin:")
        response = self.sdk.api.get_service_detail(123)
        self.assertIsInstance(response, Dict)
        self.assertIn('id', response)
        logger.info(f"Wenxin get service detail response: {response}")

    def test_get_model_version_detail(self):
        logger.info("\nTesting get model version detail for Wenxin:")
        response = self.sdk.api.get_model_version_detail(123)
        self.assertIsInstance(response, Dict)
        self.assertIn('result', response)
        logger.info(f"Wenxin get model version detail response: {response}")

    def test_publish_train_model(self):
        logger.info("\nTesting publish train model for Wenxin:")
        version_meta = {
            "description": "Test model",
            "iterationId": 1,
            "taskId": 1
        }
        response = self.sdk.api.publish_train_model(True, "Test Model", version_meta)
        self.assertIsInstance(response, Dict)
        self.assertIn('result', response)
        logger.info(f"Wenxin publish train model response: {response}")

    def test_create_training_task(self):
        logger.info("\nTesting create training task for Wenxin:")
        response = self.sdk.api.create_training_task("Test Task", "Test Description")
        self.assertIsInstance(response, Dict)
        self.assertIn('result', response)
        self.assertIn('id', response['result'])
        logger.info(f"Wenxin create training task response: {response}")

    def test_create_job_run(self):
        logger.info("\nTesting create job run for Wenxin:")
        train_config = {
            "epoch": 1,
            "learningRate": 0.00003
        }
        trainset = [{"type": 1, "id": 123}]
        response = self.sdk.api.create_job_run(1, 0, train_config, trainset, 20)
        self.assertIsInstance(response, Dict)
        self.assertIn('result', response)
        self.assertIn('id', response['result'])
        logger.info(f"Wenxin create job run response: {response}")

    def test_get_job_detail(self):
        logger.info("\nTesting get job detail for Wenxin:")
        response = self.sdk.api.get_job_detail(1, 1)
        self.assertIsInstance(response, Dict)
        self.assertIn('result', response)
        logger.info(f"Wenxin get job detail response: {response}")

    def test_stop_job(self):
        logger.info("\nTesting stop job for Wenxin:")
        response = self.sdk.api.stop_job(1, 1)
        self.assertIsInstance(response, Dict)
        self.assertIn('result', response)
        logger.info(f"Wenxin stop job response: {response}")

    def test_create_dataset(self):
        logger.info("\nTesting create dataset for Wenxin:")
        response = self.sdk.api.create_dataset("Test Dataset", 4, 20, 2000, "sysBos")
        self.assertIsInstance(response, Dict)
        self.assertIn('result', response)
        self.assertIn('id', response['result'])
        logger.info(f"Wenxin create dataset response: {response}")

    def test_release_dataset(self):
        logger.info("\nTesting release dataset for Wenxin:")
        response = self.sdk.api.release_dataset(123)
        self.assertIsInstance(response, Dict)
        self.assertIn('result', response)
        logger.info(f"Wenxin release dataset response: {response}")

    def test_import_dataset(self):
        logger.info("\nTesting import dataset for Wenxin:")
        response = self.sdk.api.import_dataset(123, False, 1, ["bos:/test/path/"])
        self.assertIsInstance(response, Dict)
        self.assertIn('result', response)
        logger.info(f"Wenxin import dataset response: {response}")

    def test_get_dataset_info(self):
        logger.info("\nTesting get dataset info for Wenxin:")
        response = self.sdk.api.get_dataset_info(123)
        self.assertIsInstance(response, Dict)
        self.assertIn('result', response)
        logger.info(f"Wenxin get dataset info response: {response}")

    def test_get_dataset_status_list(self):
        logger.info("\nTesting get dataset status list for Wenxin:")
        response = self.sdk.api.get_dataset_status_list("123,124")
        self.assertIsInstance(response, Dict)
        self.assertIn('result', response)
        logger.info(f"Wenxin get dataset status list response: {response}")

    def test_delete_dataset(self):
        logger.info("\nTesting delete dataset for Wenxin:")
        response = self.sdk.api.delete_dataset(123)
        self.assertIsInstance(response, Dict)
        self.assertIn('result', response)
        logger.info(f"Wenxin delete dataset response: {response}")

    def test_get_dataset_export_record(self):
        logger.info("\nTesting get dataset export record for Wenxin:")
        response = self.sdk.api.get_dataset_export_record(123)
        self.assertIsInstance(response, Dict)
        self.assertIn('result', response)
        logger.info(f"Wenxin get dataset export record response: {response}")

    def test_get_dataset_import_error_detail(self):
        logger.info("\nTesting get dataset import error detail for Wenxin:")
        response = self.sdk.api.get_dataset_import_error_detail(123, 1)
        self.assertIsInstance(response, Dict)
        self.assertIn('result', response)
        logger.info(f"Wenxin get dataset import error detail response: {response}")

    def test_get_prompt_template(self):
        logger.info("\nTesting get prompt template for Wenxin:")
        response = self.sdk.api.get_prompt_template(123)
        self.assertIsInstance(response, Dict)
        self.assertIn('result', response)
        logger.info(f"Wenxin get prompt template response: {response}")

    def test_weather_plugin(self):
        logger.info("\nTesting weather plugin for Wenxin:")
        response = self.sdk.api.weather_plugin("weather", "北京今天天气怎么样", ["uuid-weatherforecast"])
        self.assertIsInstance(response, Dict)
        self.assertIn('result', response)
        logger.info(f"Wenxin weather plugin response: {response}")


if __name__ == "__main__":
    unittest.main(verbosity=2)


import importlib
import os
from typing import Optional, Union, Generator, List, Dict, Any
from contextlib import contextmanager
import asyncio

from .utils.error_handler import InvokeError, InvokeModelNotFoundError, InvokeConfigError
from .models.base_api import BaseAPI
from .utils.logger import Logger
from .utils.config import Config

class OneSDK:
    def __init__(self, provider: str, credentials: Optional[dict] = None, config: Optional[Dict[str, Any]] = None):
        self.provider = provider.lower()
        self.config = Config(config or {})
        self.credentials = credentials or {}
        Logger.set_debug_mode(self.config.get('debug', False))
        self.api = self._initialize_api()
        self.current_model = None
        self._register_provider_specific_methods()

    def _initialize_api(self) -> BaseAPI:
        try:
            module = importlib.import_module(f'.models.{self.provider}.api', package=__package__)
            api_class = getattr(module, 'API')
            return api_class(self.credentials)
        except (ImportError, AttributeError) as e:
            raise InvokeConfigError(f"Unsupported or incorrectly implemented provider: {self.provider}. Error: {str(e)}")

    def _register_provider_specific_methods(self):
        for method_name in self.api.get_provider_specific_methods():
            setattr(self, method_name, self._create_proxy_method(method_name))

    def _create_proxy_method(self, method_name):
        def proxy_method(*args, **kwargs):
            return getattr(self.api, method_name)(*args, **kwargs)
        return proxy_method

    def call_provider_method(self, method_name: str, *args, **kwargs):
        if hasattr(self.api, method_name):
            return getattr(self.api, method_name)(*args, **kwargs)
        else:
            raise NotImplementedError(f"Method '{method_name}' not implemented for provider: {self.provider}")

    def list_models(self) -> List[Dict]:
        """List available models for the current provider."""
        return self.call_provider_method('list_models')

    def get_model_info(self, model_id: str) -> Dict:
        """Get information about a specific model."""
        return self.call_provider_method('get_model', model_id)

    def set_model(self, model: str):
        """Set the current model for subsequent API calls."""
        self.current_model = model
        return self

    def generate(self, model: Optional[str] = None, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]] = None,
                 **kwargs) -> Dict:
        model_to_use = model or self.current_model
        if not model_to_use:
            raise InvokeConfigError("No model specified. Either provide a model parameter or use set_model() method.")
        if messages is None:
            raise InvokeConfigError("Messages cannot be None")
        return self.call_provider_method('generate', model_to_use, messages, **kwargs)

    def stream_generate(self, model: Optional[str] = None,
                        messages: List[Dict[str, Union[str, List[Dict[str, str]]]]] = None, **kwargs) -> Generator:
        model_to_use = model or self.current_model
        if not model_to_use:
            raise InvokeConfigError("No model specified. Either provide a model parameter or use set_model() method.")
        if messages is None:
            raise InvokeConfigError("Messages cannot be None")
        yield from self.call_provider_method('stream_generate', model_to_use, messages, **kwargs)

    async def async_generate(self, model: Optional[str] = None,
                             messages: List[Dict[str, Union[str, List[Dict[str, str]]]]] = None, **kwargs) -> Dict:
        model_to_use = model or self.current_model
        if not model_to_use:
            raise InvokeConfigError("No model specified. Either provide a model parameter or use set_model() method.")
        if messages is None:
            raise InvokeConfigError("Messages cannot be None")
        return await self.call_provider_method('async_generate', model_to_use, messages, **kwargs)

    def count_tokens(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]]) -> int:
        """Count the number of tokens in the input messages for the specified model."""
        return self.call_provider_method('count_tokens', model, messages)

    def create_completion(self, model: str, prompt: str, **kwargs) -> Dict:
        """Create a completion using the legacy API (if supported by the provider)."""
        return self.call_provider_method('create_completion', model, prompt, **kwargs)

    def upload_file(self, file_path: str) -> str:
        """Upload a file and return a reference that can be used in messages."""
        return self.call_provider_method('upload_file', file_path)

    def set_proxy(self, proxy_url: str):
        """Set a proxy for API calls."""
        return self.call_provider_method('set_proxy', proxy_url)

    def get_usage(self) -> Dict:
        """Get usage statistics for the current account."""
        return self.call_provider_method('get_usage')

    @staticmethod
    def list_providers() -> List[str]:
        """List all available providers."""
        providers_dir = os.path.join(os.path.dirname(__file__), 'models')
        return [d for d in os.listdir(providers_dir)
                if os.path.isdir(os.path.join(providers_dir, d))
                and os.path.exists(os.path.join(providers_dir, d, 'api.py'))]

    def set_debug_mode(self, debug: bool):
        Logger.set_debug_mode(debug)
        self.config.set('debug', debug)

    @contextmanager
    def model_context(self, model: str):
        """Context manager for temporarily setting a model."""
        previous_model = self.current_model
        self.set_model(model)
        try:
            yield
        finally:
            self.current_model = previous_model

    def create_embedding(self, model: str, input: Union[str, List[str]], **kwargs) -> Dict:
        """Create embeddings for the given input."""
        return self.call_provider_method('create_embedding', model, input, **kwargs)

    def create_image(self, prompt: str, **kwargs) -> Dict:
        """Create an image based on the prompt."""
        return self.call_provider_method('create_image', prompt, **kwargs)

    def custom_operation(self, operation: str, **kwargs):
        """Perform a custom operation specific to the current provider."""
        return self.call_provider_method('custom_operation', operation, **kwargs)

    @property
    def cache(self):
        if not hasattr(self, '_cache'):
            self._cache = {}
        return self._cache

    def clear_cache(self):
        """Clear the internal cache."""
        self._cache.clear()


from ..base_api import BaseAPI, provider_specific
from typing import List, Dict, Union, Generator
import requests
import json
import os
from ...utils.logger import logger
from ...utils.error_handler import InvokeError, InvokeConnectionError, InvokeRateLimitError, InvokeAuthorizationError, \
    InvokeBadRequestError


class API(BaseAPI):
    BASE_URL = "https://aip.baidubce.com"

    def __init__(self, credentials: Dict[str, str]):
        super().__init__(credentials)
        self.api_key = credentials.get("api_key") or os.environ.get("WENXIN_API_KEY")
        self.secret_key = credentials.get("secret_key") or os.environ.get("WENXIN_SECRET_KEY")
        if not self.api_key or not self.secret_key:
            raise ValueError(
                "API key and secret key must be provided either in credentials or as environment variables WENXIN_API_KEY and WENXIN_SECRET_KEY")
        self.base_url = credentials.get("api_url", self.BASE_URL)
        self.access_token = self._get_access_token()
        self.session = requests.Session()
        self.session.headers.update({
            'Content-Type': 'application/json',
            'Accept': 'application/json'
        })

        # 默认的模型端点映射
        self.model_endpoints = {
            "ERNIE-Bot": "/rpc/2.0/ai_custom/v1/wenxinworkshop/chat/completions",
            "ERNIE-Bot-turbo": "/rpc/2.0/ai_custom/v1/wenxinworkshop/chat/eb-instant",
            "BLOOMZ-7B": "/rpc/2.0/ai_custom/v1/wenxinworkshop/chat/bloomz_7b1"
        }

        logger.info("Wenxin API initialized")

    def _get_access_token(self) -> str:
        """获取access_token"""
        url = f"{self.base_url}/oauth/2.0/token"
        params = {
            "grant_type": "client_credentials",
            "client_id": self.api_key,
            "client_secret": self.secret_key
        }
        try:
            response = requests.get(url, params=params)
            response_body = response.text
            try:
                response_json = response.json()
            except json.JSONDecodeError:
                response_json = {}

            response.raise_for_status()
            return response_json.get("access_token")
        except requests.RequestException as e:
            error_message = f"Error getting access token: {str(e)}\nResponse body: {response_body}"
            logger.error(error_message)
            logger.error(f"Response JSON: {json.dumps(response_json, indent=2, ensure_ascii=False)}")
            raise self._handle_error(e, additional_info=error_message)

    @provider_specific
    def set_custom_model(self, model_name: str, endpoint: str):
        """设置自定义模型及其对应的端点"""
        self.model_endpoints[model_name] = endpoint
        logger.info(f"Custom model '{model_name}' set with endpoint: {endpoint}")

    def _get_endpoint(self, model: str) -> str:
        if model in self.model_endpoints:
            return self.model_endpoints[model]
        else:
            raise ValueError(
                f"Unsupported model: {model}. Available models are: {', '.join(self.model_endpoints.keys())}")

    def generate(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]], **kwargs) -> Dict:
        """Generate a response using the specified model."""
        endpoint = self._get_endpoint(model)
        logger.info(f"Generating response with model: {model}")
        return self._call_api(endpoint, model, messages, stream=False, **kwargs)

    def stream_generate(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]],
                        **kwargs) -> Generator:
        """Generate a streaming response using the specified model."""
        endpoint = self._get_endpoint(model)
        logger.info(f"Generating streaming response with model: {model}")
        yield from self._call_api(endpoint, model, messages, stream=True, **kwargs)

    def _call_api(self, endpoint: str, model: str, messages: List[Dict], stream: bool = False, **kwargs):
        url = f"{self.base_url}{endpoint}?access_token={self.access_token}"
        payload = self._prepare_payload(model, messages, stream, **kwargs)
        headers = self.session.headers.copy()
        if stream:
            headers['Accept'] = 'text/event-stream'

        logger.debug(f"Sending request to {url}")
        logger.debug(f"Headers: {headers}")
        logger.debug(f"Payload: {json.dumps(payload, indent=2)}")

        try:
            response = self.session.post(url, json=payload, headers=headers, stream=stream)
            response.raise_for_status()

            if stream:
                logger.debug("Received streaming response")
                return self._handle_stream_response(response)
            else:
                logger.debug("Received non-streaming response")
                response_json = response.json()
                logger.debug(f"Raw API response: {json.dumps(response_json, indent=2)}")
                return self._handle_response(response_json)
        except requests.RequestException as e:
            logger.error(f"Error occurred: {str(e)}")
            raise self._handle_error(e)

    def _prepare_payload(self, model: str, messages: List[Dict], stream: bool, **kwargs):
        payload = {
            "messages": messages,
            "stream": stream
        }

        for param in ['temperature', 'top_p', 'penalty_score', 'user_id']:
            if param in kwargs:
                payload[param] = kwargs[param]

        logger.debug(f"Prepared payload: {json.dumps(payload, indent=2)}")
        return payload

    def _handle_response(self, response_data: Dict) -> Dict:
        result = {
            'id': response_data.get('id'),
            'object': response_data.get('object'),
            'created': response_data.get('created'),
            'model': response_data.get('model'),
            'choices': response_data.get('choices', []),
            'usage': response_data.get('usage', {})
        }

        # 如果 choices 为空，添加一个带有错误信息的 choice
        if not result['choices']:
            result['choices'] = [{
                'index': 0,
                'message': {'content': 'Error: No response generated'},
                'finish_reason': 'error'
            }]

        logger.debug(f"Handled response: {json.dumps(result, indent=2)}")
        return result

    def _handle_stream_chunk(self, chunk_data: Dict) -> Dict:
        result = {
            'id': chunk_data.get('id'),
            'object': chunk_data.get('object'),
            'created': chunk_data.get('created'),
            'model': chunk_data.get('model'),
            'choices': [{
                'index': 0,
                'delta': {'content': chunk_data.get('result', '')},
                'finish_reason': chunk_data.get('finish_reason')
            }],
            'usage': chunk_data.get('usage', {})
        }
        logger.debug(f"Handled stream chunk: {json.dumps(result, indent=2)}")
        return result
    def _handle_stream_response(self, response) -> Generator:
        logger.debug("Entering _handle_stream_response")
        for line in response.iter_lines():
            if line:
                logger.debug(f"Received line: {line.decode('utf-8')}")
                line = line.decode('utf-8')
                if line.startswith('data: '):
                    data = json.loads(line[6:])
                    logger.debug(f"Parsed data: {json.dumps(data, indent=2)}")
                    yield self._handle_stream_chunk(data)
        logger.debug("Exiting _handle_stream_response")

    def _handle_error(self, error: requests.RequestException, additional_info: str = "") -> InvokeError:
        error_message = f"{str(error)}\n{additional_info}"
        if isinstance(error, requests.ConnectionError):
            logger.error(f"Connection error: {error_message}")
            return InvokeConnectionError(error_message)
        elif isinstance(error, requests.Timeout):
            logger.error(f"Timeout error: {error_message}")
            return InvokeConnectionError(error_message)
        elif isinstance(error, requests.HTTPError):
            if error.response.status_code == 429:
                logger.error(f"Rate limit error: {error_message}")
                return InvokeRateLimitError(error_message)
            elif error.response.status_code in (401, 403):
                logger.error(f"Authorization error: {error_message}")
                return InvokeAuthorizationError(error_message)
            elif error.response.status_code >= 500:
                logger.error(f"Server unavailable error: {error_message}")
                return InvokeServerUnavailableError(error_message)
            else:
                logger.error(f"Bad request error: {error_message}")
                return InvokeBadRequestError(error_message)
        else:
            logger.error(f"Unknown error: {error_message}")
            return InvokeError(error_message)

    def count_tokens(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]]) -> int:
        """Count tokens in a message."""
        token_count = sum(len(str(message.get('content', '')).split()) for message in messages)
        logger.info(f"Estimated token count for model {model}: {token_count}")
        return token_count

    def set_proxy(self, proxy_url: str):
        """Set a proxy for API calls."""
        self.session.proxies = {
            'http': proxy_url,
            'https': proxy_url
        }
        logger.info(f"Proxy set to {proxy_url}")


from ..base_api import BaseAPI, provider_specific
from typing import List, Dict, Union, Generator
import requests
import json
import os
import base64
from urllib.parse import urljoin
from ...utils.logger import logger
from ...utils.error_handler import InvokeError, InvokeConnectionError, InvokeRateLimitError, InvokeAuthorizationError, \
    InvokeBadRequestError


class API(BaseAPI):
    BASE_URL = "https://api.anthropic.com"
    API_VERSION = "2023-06-01"

    def __init__(self, credentials: Dict[str, str]):
        super().__init__(credentials)
        self.api_key = credentials.get("api_key") or os.environ.get("ANTHROPIC_API_KEY")
        if not self.api_key:
            raise ValueError(
                "API key must be provided either in credentials or as an environment variable ANTHROPIC_API_KEY")
        self.base_url = credentials.get("api_url", self.BASE_URL)
        self.session = requests.Session()
        self.session.headers.update({
            'X-API-Key': self.api_key,
            'anthropic-version': self.API_VERSION,
            'Content-Type': 'application/json'
        })
        logger.info("Anthropic API initialized")

    @provider_specific
    def list_models(self) -> List[Dict]:
        """List available models."""
        logger.info("Fetching available models")
        models = self._call_api("/v1/models", method="GET")
        logger.info(f"Available models: {[model['id'] for model in models['data']]}")
        return models

    @provider_specific
    def get_model(self, model_id: str) -> Dict:
        """Get information about a specific model."""
        logger.info(f"Fetching information for model: {model_id}")
        model_info = self._call_api(f"/v1/models/{model_id}", method="GET")
        logger.info(f"Model info for {model_id}: {model_info}")
        return model_info

    def generate(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]], **kwargs) -> Dict:
        """Generate a response using the specified model."""
        logger.info(f"Generating response with model: {model}")
        max_tokens = kwargs.pop('max_tokens', 1000)  # 默认值设为1000，可以根据需要调整
        return self._call_api("/v1/messages", model=model, messages=messages, max_tokens=max_tokens, stream=False,
                              **kwargs)

    def stream_generate(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]],
                        **kwargs) -> Generator:
        """Generate a streaming response using the specified model."""
        logger.info(f"Generating streaming response with model: {model}")
        max_tokens = kwargs.pop('max_tokens', 1000)
        response = self._call_api("/v1/messages", model=model, messages=messages, max_tokens=max_tokens, stream=True,
                                  **kwargs)
        for chunk in response:
            if 'content' in chunk:
                for content_item in chunk['content']:
                    if content_item['type'] == 'text':
                        yield {'delta': {'text': content_item['text']}}

    def count_tokens(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]]) -> int:
        """Count tokens in a message."""
        logger.info(f"Counting tokens for model: {model}")
        response = self._call_api("/v1/messages", model=model, messages=messages, max_tokens=1)
        token_count = response.get('usage', {}).get('input_tokens', 0)
        logger.info(f"Token count for model {model}: {token_count}")
        return token_count

    def _call_api(self, endpoint: str, **kwargs) -> Union[Dict, Generator]:
        url = urljoin(self.base_url, endpoint)
        headers = self.session.headers.copy()
        method = kwargs.pop('method', 'POST')
        stream = kwargs.pop('stream', False)

        if stream:
            headers['Accept'] = 'text/event-stream'

        try:
            payload = self._prepare_payload(**kwargs) if method == 'POST' else None
            logger.debug(f"Sending request to {url}")
            logger.debug(f"Headers: {headers}")
            if payload:
                logger.debug(f"Payload: {json.dumps(payload, indent=2)}")

            response = self.session.request(method, url, json=payload, headers=headers, stream=stream)

            # 打印响应状态码和头部
            logger.debug(f"Response status code: {response.status_code}")
            logger.debug(f"Response headers: {response.headers}")

            # 尝试打印响应体，即使状态码不是 200
            try:
                if not stream:
                    response_body = response.json()
                    logger.debug(f"Response body: {json.dumps(response_body, indent=2)}")
                else:
                    logger.debug("Streaming response, body not available")
            except json.JSONDecodeError:
                logger.debug(f"Response body (not JSON): {response.text}")

            response.raise_for_status()

            if stream:
                return self._handle_stream_response(response)
            else:
                return response.json()
        except requests.RequestException as e:
            logger.error(f"Request failed: {str(e)}")
            if hasattr(e, 'response') and e.response is not None:
                logger.error(f"Error response status code: {e.response.status_code}")
                logger.error(f"Error response headers: {e.response.headers}")
                try:
                    error_body = e.response.json()
                    logger.error(f"Error response body: {json.dumps(error_body, indent=2)}")
                except json.JSONDecodeError:
                    logger.error(f"Error response body (not JSON): {e.response.text}")
            raise self._handle_request_error(e)

    def _prepare_payload(self, **kwargs) -> Dict:
        payload = {
            "model": kwargs.pop('model'),
            "messages": self._process_messages(kwargs.pop('messages', [])),
            "max_tokens": kwargs.pop('max_tokens', 1),
        }
        allowed_params = ['temperature', 'top_p', 'stop', 'stream', 'metadata']
        payload.update({k: v for k, v in kwargs.items() if k in allowed_params})
        return payload

    def _process_messages(self, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]]) -> List[Dict]:
        processed_messages = []
        for message in messages:
            if isinstance(message.get('content'), list):
                processed_content = []
                for content in message['content']:
                    if content.get('type') == 'image':
                        processed_content.append(self._process_image_content(content))
                    else:
                        processed_content.append(content)
                message['content'] = processed_content
            processed_messages.append(message)
        return processed_messages

    def _process_image_content(self, content: Dict) -> Dict:
        if content.get('source', {}).get('type') == 'path':
            with open(content['source']['path'], 'rb') as image_file:
                base64_image = base64.b64encode(image_file.read()).decode('utf-8')
            content['source'] = {
                'type': 'base64',
                'media_type': content['source'].get('media_type', 'image/jpeg'),
                'data': base64_image
            }
        return content

    def _handle_stream_response(self, response: requests.Response) -> Generator:
        for line in response.iter_lines():
            if line:
                line = line.decode('utf-8')
                logger.debug(f"Received line: {line}")
                try:
                    data = json.loads(line)
                    logger.debug(f"Parsed data: {data}")
                    yield data
                except json.JSONDecodeError:
                    logger.error(f"Failed to parse streaming response: {line}")

    def _handle_request_error(self, error: requests.RequestException) -> InvokeError:
        if isinstance(error, requests.ConnectionError):
            return InvokeConnectionError(str(error))
        elif isinstance(error, requests.Timeout):
            return InvokeConnectionError(str(error))
        elif isinstance(error, requests.HTTPError):
            if error.response.status_code == 429:
                return InvokeRateLimitError(str(error))
            elif error.response.status_code in (401, 403):
                return InvokeAuthorizationError(str(error))
            else:
                return InvokeBadRequestError(str(error))
        else:
            return InvokeError(str(error))

    def set_proxy(self, proxy_url: str):
        """Set a proxy for API calls."""
        self.session.proxies = {
            'http': proxy_url,
            'https': proxy_url
        }
        logger.info(f"Proxy set to {proxy_url}")



import os
import sys
import unittest
from typing import List, Dict
import time
from unittest.mock import patch
import io

# 添加项目根目录到 Python 路径
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from llm_onesdk.core import OneSDK
from llm_onesdk.utils.error_handler import InvokeError, InvokeConnectionError, InvokeRateLimitError, InvokeAuthorizationError, InvokeBadRequestError
from llm_onesdk.utils.logger import Logger, logger


class TestMiniMaxAPI(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        Logger.set_debug_mode(True)
        logger.info("Setting up TestMiniMaxAPI class")

        cls.api_key = os.environ.get("MINIMAX_API_KEY")
        cls.group_id = os.environ.get("MINIMAX_GROUP_ID")
        if not cls.api_key or not cls.group_id:
            raise ValueError("Please set MINIMAX_API_KEY and MINIMAX_GROUP_ID environment variables")

        cls.sdk = OneSDK("minimax", {
            "api_key": cls.api_key,
            "group_id": cls.group_id
        })
        cls.sdk.set_debug_mode(True)

        cls.default_model = "abab5.5-chat"  # 使用 MiniMax 的默认模型

    def setUp(self):
        time.sleep(1)  # 添加延迟以避免频率限制

    def test_generate(self):
        logger.info("\nTesting generate for MiniMax:")
        messages = [{"role": "user", "content": "Count from 1 to 5."}]
        bot_setting = [{"bot_name": "MM智能助理", "content": "MM智能助理是一款由MiniMax自研的大型语言模型。"}]
        response = self.sdk.generate(self.default_model, messages, bot_setting=bot_setting)
        self.assertIsInstance(response, Dict)
        self.assertIn('choices', response)
        self.assertIn('message', response['choices'][0])
        self.assertIn('content', response['choices'][0]['message'])
        logger.info(f"MiniMax response: {response['choices'][0]['message']['content']}")

    def test_stream_generate(self):
        logger.info("\nTesting stream_generate for MiniMax:")
        messages = [{"role": "user", "content": "Count from 1 to 5."}]
        bot_setting = [{"bot_name": "MM智能助理", "content": "MM智能助理是一款由MiniMax自研的大型语言模型。"}]
        stream = self.sdk.stream_generate(model=self.default_model, messages=messages, bot_setting=bot_setting)
        full_response = ""
        chunk_count = 0
        start_time = time.time()
        timeout = 30  # 30 seconds timeout
        for chunk in stream:
            if time.time() - start_time > timeout:
                logger.warning("Timeout reached for MiniMax")
                break
            chunk_count += 1
            self.assertIsInstance(chunk, Dict)
            self.assertIn('delta', chunk)
            content = chunk['delta'].get('text', '')
            if content:
                full_response += content
                logger.info(f"MiniMax chunk {chunk_count}: {content}")
        logger.info(f"\nMiniMax full response: {full_response}")
        logger.info(f"Total chunks received: {chunk_count}")
        logger.info(f"Time taken: {time.time() - start_time:.2f} seconds")
        self.assertTrue(chunk_count > 0, "No chunks were received")
        self.assertTrue(len(full_response) > 0, "No content was received")

    def test_create_embedding(self):
        logger.info("\nTesting create_embedding for MiniMax:")
        input_text = ["Hello, world!"]
        response = self.sdk.api.create_embedding("embo-01", input_text, type="db")
        self.assertIsInstance(response, Dict)
        self.assertIn('vectors', response)
        self.assertIsInstance(response['vectors'], List)
        self.assertEqual(len(response['vectors']), 1)  # 应该只有一个向量
        self.assertEqual(len(response['vectors'][0]), 1536)  # 向量长度应为 1536
        logger.info(f"MiniMax embedding: {response['vectors'][0][:5]}...")

    def test_text_to_speech(self):
        logger.info("\nTesting text_to_speech for MiniMax:")
        text = "Hello, how are you?"
        voice_setting = {
            "voice_id": "male-qn-qingse",
            "speed": 1,
            "vol": 1,
            "pitch": 0,
            "emotion": "happy"
        }
        audio_setting = {
            "sample_rate": 32000,
            "bitrate": 128000,
            "format": "mp3"
        }
        model = "speech-01-turbo"
        response = self.sdk.api.text_to_speech(model, text, voice_setting=voice_setting, audio_setting=audio_setting)
        self.assertIsInstance(response, Dict)
        self.assertIn('data', response)
        self.assertIn('audio', response['data'])
        logger.info("Text-to-speech conversion successful")

    def test_create_video_generation_task(self):
        logger.info("\nTesting create_video_generation_task for MiniMax:")
        model = "video-01"
        prompt = "A beautiful sunset over the ocean"
        response = self.sdk.api.create_video_generation_task(model, prompt)
        self.assertIsInstance(response, Dict)
        if response['base_resp']['status_code'] == 1002:
            logger.warning("Rate limit reached, skipping task ID assertion")
        else:
            self.assertIn('task_id', response)
        logger.info(f"Video generation task response: {response}")

    def test_query_video_generation_task(self):
        logger.info("\nTesting query_video_generation_task for MiniMax:")
        # First, create a task
        model = "video-01"
        prompt = "A beautiful sunset over the ocean"
        create_response = self.sdk.api.create_video_generation_task(model, prompt)
        if create_response['base_resp']['status_code'] == 1002:
            logger.warning("Rate limit reached, skipping query test")
            return
        task_id = create_response['task_id']

        # Now query the task
        query_response = self.sdk.api.query_video_generation_task(task_id)
        self.assertIsInstance(query_response, Dict)
        self.assertIn('status', query_response)
        logger.info(f"Video generation task status: {query_response['status']}")

    def test_upload_music(self):
        logger.info("\nTesting upload_music for MiniMax:")
        test_file_content = b"This is a test audio file content."
        test_file = io.BytesIO(test_file_content)
        test_file.name = "test_audio.mp3"

        try:
            response = self.sdk.api.upload_music(test_file)
            self.assertIsInstance(response, Dict)
            self.assertIn('voice_id', response)
            self.assertIn('instrumental_id', response)
            logger.info(
                f"Music uploaded successfully. Voice ID: {response['voice_id']}, Instrumental ID: {response['instrumental_id']}")
        except InvokeError as e:
            self.fail(f"Music upload failed: {str(e)}")

    def test_generate_music(self):
        logger.info("\nTesting generate_music for MiniMax:")
        model = "music-01"
        lyrics = "This is a test song\nWith some simple lyrics\n"
        response = self.sdk.api.generate_music(model, lyrics)
        self.assertIsInstance(response, Dict)
        self.assertIn('data', response)
        logger.info("Music generation successful")

    def test_file_operations(self):
        logger.info("\nTesting file operations for MiniMax:")

        # 测试上传文件
        test_file_content = b"This is a test file content."
        test_file = io.BytesIO(test_file_content)
        test_file.name = "test.txt"  # 给文件一个名字
        upload_response = self.sdk.api.upload_file(test_file, "retrieval")
        self.assertIsInstance(upload_response, Dict)
        self.assertIn('file', upload_response)
        self.assertIn('file_id', upload_response['file'])
        file_id = upload_response['file']['file_id']
        logger.info(f"File uploaded successfully. File ID: {file_id}")

        # 测试获取文件信息
        file_info = self.sdk.api.get_file_info(file_id)
        self.assertIsInstance(file_info, Dict)
        self.assertIn('file', file_info)
        self.assertEqual(file_info['file']['file_id'], file_id)
        logger.info(f"File info retrieved successfully: {file_info}")

        # 测试获取文件内容
        file_content = self.sdk.api.get_file_content(file_id)
        logger.debug(f"Retrieved file content: {file_content}")  # 只记录前100个字节
        if isinstance(file_content, dict):
            # 如果返回的是字典，可能是错误消息
            self.fail(f"Failed to retrieve file content: {file_content}")
        else:
            # 如果是字节串，应该是文件内容
            self.assertEqual(file_content, test_file_content)
            logger.info("File content retrieved successfully")

        # 测试列出文件
        files = self.sdk.api.list_files(purpose="retrieval")
        self.assertIsInstance(files, List)
        self.assertTrue(any(file['file_id'] == file_id for file in files))
        logger.info(f"Listed {len(files)} files")

        # 测试删除文件
        delete_response = self.sdk.api.delete_file(file_id)
        self.assertIsInstance(delete_response, Dict)
        self.assertEqual(delete_response['file_id'], file_id)
        logger.info("File deleted successfully")

    def test_knowledge_base_operations(self):
        logger.info("\nTesting knowledge base operations for MiniMax:")

        # 创建知识库
        kb_name = f"Test KB {int(time.time())}"
        operator_id = int(time.time())
        create_kb_response = self.sdk.api.create_knowledge_base(kb_name, "kbq-001", operator_id)
        self.assertIsInstance(create_kb_response, Dict)
        self.assertIn('knowledge_base_id', create_kb_response)
        kb_id = create_kb_response['knowledge_base_id']
        logger.info(f"Knowledge base created successfully. ID: {kb_id}")

        # 获取知识库信息
        kb_info = self.sdk.api.get_knowledge_base(kb_id)
        self.assertIsInstance(kb_info, Dict)
        self.assertIn('knowledge_base', kb_info)
        self.assertEqual(kb_info['knowledge_base']['knowledge_base_id'], kb_id)
        logger.info(f"Knowledge base info retrieved successfully: {kb_info}")

        # 列出知识库
        kb_list = self.sdk.api.list_knowledge_bases()
        self.assertIsInstance(kb_list, Dict)
        self.assertIn('knowledge_bases', kb_list)
        self.assertTrue(any(kb['knowledge_base_id'] == kb_id for kb in kb_list['knowledge_bases']))
        logger.info(f"Listed {len(kb_list['knowledge_bases'])} knowledge bases")

        # 删除知识库
        delete_kb_response = self.sdk.api.delete_knowledge_base(kb_id, operator_id)
        self.assertIsInstance(delete_kb_response, Dict)
        self.assertIn('base_resp', delete_kb_response)
        self.assertEqual(delete_kb_response['base_resp']['status_code'], 0)
        logger.info("Knowledge base deleted successfully")

    def test_chatcompletion_pro(self):
        logger.info("\nTesting chatcompletion_pro for MiniMax:")
        messages = [{"sender_type": "USER", "sender_name": "User", "text": "Count from 1 to 5."}]
        bot_setting = [{"bot_name": "MM智能助理", "content": "MM智能助理是一款由MiniMax自研的大型语言模型。"}]
        reply_constraints = {"sender_type": "BOT", "sender_name": "MM智能助理"}
        response = self.sdk.api.chatcompletion_pro(
            self.default_model,
            messages,
            bot_setting=bot_setting,
            reply_constraints=reply_constraints
        )
        self.assertIsInstance(response, Dict)
        self.assertIn('choices', response)
        self.assertIn('messages', response['choices'][0])
        self.assertIn('text', response['choices'][0]['messages'][0])
        logger.info(f"MiniMax ChatCompletion Pro response: {response['choices'][0]['messages'][0]['text']}")

    def test_stream_chatcompletion_pro(self):
        logger.info("\nTesting stream_chatcompletion_pro for MiniMax:")
        messages = [{"sender_type": "USER", "sender_name": "小明", "text": "帮我用英文翻译下面这句话：我来自中国"}]
        stream = self.sdk.api.stream_chatcompletion_pro(self.default_model, messages)
        full_response = []
        chunk_count = 0
        start_time = time.time()
        timeout = 30  # 30 seconds timeout
        for chunk in stream:
            if time.time() - start_time > timeout:
                logger.warning("Timeout reached for MiniMax")
                break
            chunk_count += 1
            self.assertIsInstance(chunk, Dict)
            self.assertIn('delta', chunk)
            delta_content = chunk['delta']
            full_response.extend(delta_content)
            logger.info(f"MiniMax chunk {chunk_count}: {delta_content}")
        logger.info(f"\nMiniMax full response: {full_response}")
        logger.info(f"Total chunks received: {chunk_count}")
        logger.info(f"Time taken: {time.time() - start_time:.2f} seconds")
        self.assertTrue(chunk_count > 0, "No chunks were received")
        self.assertTrue(len(full_response) > 0, "No content was received")

    def test_create_long_speech_task(self):
        logger.info("\nTesting create_long_speech_task for MiniMax:")
        model = "speech-01-turbo"
        text = "This is a long text for speech synthesis. " * 10
        voice_setting = {
            "voice_id": "male-qn-qingse",
            "speed": 1,
            "vol": 1,
            "pitch": 0
        }
        audio_setting = {
            "audio_sample_rate": 32000,
            "bitrate": 128000,
            "format": "mp3",
            "channel": 2
        }
        response = self.sdk.api.create_long_speech_task(model, text, voice_setting, audio_setting)
        self.assertIsInstance(response, Dict)
        self.assertIn('task_id', response)
        logger.info(f"Long speech task created successfully. Task ID: {response['task_id']}")

    def test_query_long_speech_task(self):
        logger.info("\nTesting query_long_speech_task for MiniMax:")
        # First, create a task
        model = "speech-01-turbo"
        text = "This is a long text for speech synthesis. " * 10
        voice_setting = {
            "voice_id": "male-qn-qingse",
            "speed": 1,
            "vol": 1,
            "pitch": 0
        }
        audio_setting = {
            "audio_sample_rate": 32000,
            "bitrate": 128000,
            "format": "mp3",
            "channel": 2
        }
        create_response = self.sdk.api.create_long_speech_task(model, text, voice_setting, audio_setting)
        task_id = create_response['task_id']

        # Now query the task
        query_response = self.sdk.api.query_long_speech_task(task_id)
        self.assertIsInstance(query_response, Dict)
        self.assertIn('status', query_response)
        logger.info(f"Long speech task status: {query_response['status']}")

    def test_voice_cloning(self):
        logger.info("\nTesting voice_cloning for MiniMax:")
        # First, upload a file for voice cloning
        test_file_content = b"This is a test audio file content for voice cloning."
        test_file = io.BytesIO(test_file_content)
        test_file.name = "test_voice_clone.mp3"
        upload_response = self.sdk.api.upload_file(test_file, "voice_clone")
        file_id = upload_response['file']['file_id']

        # Now clone the voice
        voice_id = f"test_voice_{int(time.time())}"
        response = self.sdk.api.voice_cloning(file_id, voice_id)
        self.assertIsInstance(response, Dict)
        self.assertIn('base_resp', response)
        status_code = response['base_resp']['status_code']
        self.assertTrue(status_code == 0 or status_code == 2038,
                        f"Unexpected status code: {status_code}")
        if status_code == 0:
            logger.info(f"Voice cloned successfully. Voice ID: {voice_id}")
        else:
            logger.warning(f"Voice cloning failed with status code: {status_code}")

    def test_text_to_voice(self):
        logger.info("\nTesting text_to_voice for MiniMax:")
        gender = "female"
        age = "young"
        voice_desc = ["Kind and friendly", "Calm tone"]
        text = "Hello, this is a test for text to voice generation."
        response = self.sdk.api.text_to_voice(gender, age, voice_desc, text)
        self.assertIsInstance(response, Dict)
        self.assertIn('voice_id', response)
        self.assertIn('trial_audio', response)
        logger.info(f"Text to voice generated successfully. Voice ID: {response['voice_id']}")

    def test_delete_voice(self):
        logger.info("\nTesting delete_voice for MiniMax:")
        # First, create a voice using text_to_voice
        gender = "male"
        age = "middle-aged"
        voice_desc = ["Deep voice", "Serious tone"]
        text = "This is a test voice for deletion."
        create_response = self.sdk.api.text_to_voice(gender, age, voice_desc, text)
        voice_id = create_response['voice_id']

        # Now delete the voice
        delete_response = self.sdk.api.delete_voice("voice_generation", voice_id)
        self.assertIsInstance(delete_response, Dict)
        self.assertIn('base_resp', delete_response)
        self.assertEqual(delete_response['base_resp']['status_code'], 0)
        logger.info(f"Voice deleted successfully. Voice ID: {voice_id}")

    @patch.object(OneSDK, 'generate')
    def test_error_handling_generate(self, mock_generate):
        mock_generate.side_effect = InvokeError("Test error")
        with self.assertRaises(InvokeError):
            self.sdk.generate(self.default_model, [{"role": "user", "content": "Test"}])

    @patch.object(OneSDK, 'generate')
    def test_connection_error(self, mock_generate):
        mock_generate.side_effect = InvokeConnectionError("Connection error")
        with self.assertRaises(InvokeConnectionError):
            self.sdk.generate(self.default_model, [{"role": "user", "content": "Test"}])

    @patch.object(OneSDK, 'generate')
    def test_rate_limit_error(self, mock_generate):
        mock_generate.side_effect = InvokeRateLimitError("Rate limit exceeded")
        with self.assertRaises(InvokeRateLimitError):
            self.sdk.generate(self.default_model, [{"role": "user", "content": "Test"}])

    @patch.object(OneSDK, 'generate')
    def test_authorization_error(self, mock_generate):
        mock_generate.side_effect = InvokeAuthorizationError("Invalid API key")
        with self.assertRaises(InvokeAuthorizationError):
            self.sdk.generate(self.default_model, [{"role": "user", "content": "Test"}])

    def test_set_proxy(self):
        logger.info("\nTesting set_proxy for MiniMax:")
        proxy_url = "http://example.com:8080"  # 使用一个示例 URL，不要实际连接
        self.sdk.set_proxy(proxy_url)
        logger.info(f"Proxy set to {proxy_url}")
        # 注意：这里我们只是测试方法调用，不测试实际连接


if __name__ == "__main__":
    unittest.main(verbosity=2)


from ..base_api import BaseAPI, provider_specific
from typing import List, Dict, Union, Generator
import requests
import json
import os
from urllib.parse import urljoin
from ...utils.logger import logger
from ...utils.error_handler import (
    InvokeError,
    InvokeConnectionError,
    InvokeServerUnavailableError,
    InvokeRateLimitError,
    InvokeAuthorizationError,
    InvokeBadRequestError,
    InvokeUnsupportedOperationError
)


class API(BaseAPI):
    BASE_URL = "https://ark.cn-beijing.volces.com/api/v3/"

    def __init__(self, credentials: Dict[str, str]):
        super().__init__(credentials)
        self.api_key = credentials.get("api_key") or os.environ.get("DOUBAO_API_KEY")
        if not self.api_key:
            raise ValueError(
                "API key must be provided either in credentials or as an environment variable DOUBAO_API_KEY")
        self.base_url = credentials.get("api_url", self.BASE_URL)
        self.session = requests.Session()
        self.session.headers.update({
            'Authorization': f'Bearer {self.api_key}',
            'Content-Type': 'application/json'
        })
        logger.info("Doubao API initialized")

    def list_models(self) -> List[Dict]:
        """List available models for Doubao."""
        # Implement if Doubao supports listing models, otherwise:
        raise InvokeUnsupportedOperationError("Listing models is not supported by Doubao API")

    def get_model(self, model_id: str) -> Dict:
        """Get information about a specific model."""
        # Implement if Doubao supports getting model info, otherwise:
        raise InvokeUnsupportedOperationError("Getting model information is not supported by Doubao API")

    def generate(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]], **kwargs) -> Dict:
        """Generate a response using the specified model."""
        logger.info(f"Generating response with model: {model}")
        payload = {
            "model": model,
            "messages": messages,
            "stream": kwargs.get("stream", False),
            "max_tokens": kwargs.get("max_tokens", 4096),
        }
        optional_params = ["stream_options", "stop", "frequency_penalty", "presence_penalty",
                           "temperature", "top_p", "logprobs", "top_logprobs", "logit_bias"]
        for param in optional_params:
            if param in kwargs:
                payload[param] = kwargs[param]
        return self._call_api("chat/completions", **payload)

    @provider_specific
    def tokenize(self, model: str, text: Union[str, List[str]]) -> Dict:
        """
        Tokenize the given text using the specified model.

        Args:
            model (str): The model to use for tokenization.
            text (Union[str, List[str]]): The text or list of texts to tokenize.

        Returns:
            Dict: A dictionary containing the tokenization results.
        """
        logger.info(f"Tokenizing text with model: {model}")

        if isinstance(text, str):
            text = [text]

        payload = {
            "model": model,
            "text": text
        }

        try:
            response = self._call_api("tokenization", **payload)
            if 'data' not in response or not response['data']:
                raise InvokeError("Unexpected response format from tokenization API")

            logger.info(f"Tokenization completed for {len(text)} text(s)")
            return response
        except Exception as e:
            logger.error(f"Error in tokenize: {str(e)}")
            raise

    def stream_generate(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]],
                        **kwargs) -> Generator:
        """Generate a streaming response using the specified model."""
        logger.info(f"Generating streaming response with model: {model}")
        kwargs['stream'] = True
        return self._call_api("chat/completions", model=model, messages=messages, **kwargs)

    def create_embedding(self, model: str, input: Union[str, List[str]], **kwargs) -> Dict:
        """Create embeddings for the given input."""
        logger.info(f"Creating embedding with model: {model}")
        payload = {
            "model": model,
            "input": input,
            "encoding_format": kwargs.get("encoding_format", "float")
        }
        return self._call_api("embeddings", **payload)

    def count_tokens(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]]) -> int:
        """Count tokens in messages."""
        logger.info(f"Counting tokens for model: {model}")

        # 提取消息内容，形成文本列表
        text_list = [msg["content"] for msg in messages if isinstance(msg.get("content"), str)]

        payload = {
            "model": model,
            "text": text_list
        }

        try:
            response = self._call_api("tokenization", **payload)
            if 'data' not in response or not response['data']:
                raise InvokeError("Unexpected response format from tokenization API")

            # 计算所有文本的总 token 数
            token_count = sum(item.get('total_tokens', 0) for item in response['data'])
            logger.info(f"Token count for model {model}: {token_count}")
            return token_count
        except Exception as e:
            logger.error(f"Error in count_tokens: {str(e)}")
            return self._fallback_count_tokens(text_list)

    def _fallback_count_tokens(self, text_list: List[str]) -> int:
        """A simple fallback method to estimate token count."""
        total_chars = sum(len(text) for text in text_list)
        # 假设平均每个token包含4个字符
        estimated_tokens = total_chars // 4
        logger.info(f"Estimated token count (fallback method): {estimated_tokens}")
        return estimated_tokens

    @provider_specific
    def create_context(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]], **kwargs) -> Dict:
        """Create a context for caching."""
        logger.info(f"Creating context for model: {model}")
        payload = {
            "model": model,
            "messages": messages,
            "mode": kwargs.get("mode", "session"),
            "ttl": kwargs.get("ttl", 86400),
        }
        if "truncation_strategy" in kwargs:
            payload["truncation_strategy"] = kwargs["truncation_strategy"]
        return self._call_api("context/create", **payload)

    @provider_specific
    def generate_with_context(self, model: str, context_id: str,
                              messages: List[Dict[str, Union[str, List[Dict[str, str]]]]], **kwargs) -> Dict:
        """Generate a response using a context."""
        logger.info(f"Generating response with context for model: {model}")
        payload = {
            "model": model,
            "context_id": context_id,
            "messages": messages,
        }
        optional_params = ["stream", "stream_options", "max_tokens", "stop", "temperature",
                           "top_p", "logprobs", "top_logprobs", "logit_bias"]
        for param in optional_params:
            if param in kwargs:
                payload[param] = kwargs[param]
        return self._call_api("context/chat/completions", **payload)

    @provider_specific
    def visual_generate(self, model: str, messages: List[Dict[str, Union[str, List[Dict[str, str]]]]],
                        **kwargs) -> Dict:
        """Generate a response using visual understanding."""
        logger.info(f"Generating visual response with model: {model}")
        payload = {
            "model": model,
            "messages": messages,
        }
        optional_params = ["stream", "stream_options", "max_tokens", "stop", "temperature",
                           "top_p", "logprobs", "top_logprobs", "logit_bias"]
        for param in optional_params:
            if param in kwargs:
                payload[param] = kwargs[param]
        return self._call_api("chat/completions", **payload)

    def _call_api(self, endpoint: str, **kwargs):
        url = urljoin(self.base_url, endpoint)
        method = kwargs.pop('method', 'POST')
        stream = kwargs.pop('stream', False)

        headers = {
            'Authorization': f'Bearer {self.api_key}',
            'Content-Type': 'application/json',
        }

        logger.debug(f"Sending request to {url}")
        logger.debug(f"Method: {method}")
        logger.debug(f"Headers: {headers}")
        logger.debug(f"Kwargs: {kwargs}")

        try:
            if method == "GET":
                response = self.session.get(url, headers=headers, params=kwargs)
            else:
                response = self.session.post(url, headers=headers, json=kwargs, stream=stream)

            response.raise_for_status()

            if stream:
                return self._handle_stream_response(response)
            else:
                return response.json()
        except requests.RequestException as e:
            logger.error(f"API call error: {str(e)}")
            error_message = "No error message provided"
            if e.response is not None:
                try:
                    error_json = e.response.json()
                    if 'error' in error_json and 'message' in error_json['error']:
                        error_message = error_json['error']['message']
                except json.JSONDecodeError:
                    error_message = e.response.text

            logger.error(f"Error message: {error_message}")
            logger.error(f"Response status code: {e.response.status_code if e.response else 'N/A'}")
            logger.error(f"Response content: {e.response.text if e.response else 'N/A'}")
            raise self._handle_error(e, error_message)

    def _handle_stream_response(self, response) -> Generator:
        logger.debug("Entering _handle_stream_response")
        for line in response.iter_lines():
            if line:
                logger.debug(f"Received line: {line.decode('utf-8')}")
                yield json.loads(line.decode('utf-8'))
        logger.debug("Exiting _handle_stream_response")

    def _handle_error(self, error: requests.RequestException, error_message: str) -> InvokeError:
        if isinstance(error, requests.ConnectionError):
            return InvokeConnectionError(f"{str(error)}. Error message: {error_message}")
        elif isinstance(error, requests.Timeout):
            return InvokeConnectionError(f"{str(error)}. Error message: {error_message}")
        elif isinstance(error, requests.HTTPError):
            if error.response.status_code == 429:
                return InvokeRateLimitError(f"{str(error)}. Error message: {error_message}")
            elif error.response.status_code in (401, 403):
                return InvokeAuthorizationError(f"{str(error)}. Error message: {error_message}")
            elif error.response.status_code >= 500:
                return InvokeServerUnavailableError(f"{str(error)}. Error message: {error_message}")
            else:
                return InvokeBadRequestError(f"{str(error)}. Error message: {error_message}")
        else:
            return InvokeError(f"{str(error)}. Error message: {error_message}")

    def set_proxy(self, proxy_url: str):
        """Set a proxy for API calls."""
        self.session.proxies = {
            'http': proxy_url,
            'https': proxy_url
        }
        logger.info(f"Proxy set to {proxy_url}")



from setuptools import setup, find_packages

setup(
    name="llm_onesdk",
    version="0.0.5",
    author="anycodes",
    author_email="liuyu@xmail.tech",
    description="OneSDK is a Python library that provides a unified interface for interacting with various Large Language Model (LLM) providers.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://onesdk.llmpages.cn/",
    packages=find_packages(exclude=["tests*"]),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
    install_requires=[
        "requests>=2.25.1,<3",
        "python-dotenv>=0.19.0,<1",
        "typing-extensions>=3.7.4,<5",
    ],
    extras_require={
        "test": ["unittest2>=1.1.0"],
    },
    project_urls={
        "GitHub": "https://github.com/LLMPages/onesdk",
    },
    license="MIT",
    keywords="LLM API SDK NLP AI",
    include_package_data=True,
)